@@ -300,10 +300,10 @@ thunar_folder_finalize (GObject *object)
     }
 
   /* release references to the new files */
-  thunar_file_list_free (folder->new_files);
+  thunar_g_file_list_free (folder->new_files);
 
   /* release references to the current files */
-  thunar_file_list_free (folder->files);
+  thunar_g_file_list_free (folder->files);
 
   (*G_OBJECT_CLASS (thunar_folder_parent_class)->finalize) (object);
 }
@@ -467,11 +467,11 @@ thunar_folder_finished (ExoJob       *job,
           g_signal_emit (G_OBJECT (folder), folder_signals[FILES_REMOVED], 0, files);
 
           /* release the removed files list */
-          thunar_file_list_free (files);
+          thunar_g_file_list_free (files);
         }
 
       /* drop the temporary new_files list */
-      thunar_file_list_free (folder->new_files);
+      thunar_g_file_list_free (folder->new_files);
       folder->new_files = NULL;
     }
   else
@@ -636,6 +636,8 @@ thunar_folder_monitor (GFileMonitor     *monitor,
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
   _thunar_return_if_fail (folder->monitor == monitor);
   _thunar_return_if_fail (folder->job == NULL);
+  _thunar_return_if_fail (THUNAR_IS_FILE (folder->corresponding_file));
+  _thunar_return_if_fail (G_IS_FILE (event_file));
 
   /* check on which file the event occurred */
   if (!g_file_equal (event_file, thunar_file_get_file (folder->corresponding_file)))
@@ -832,7 +834,7 @@ thunar_folder_reload (ThunarFolder *folder)
     }
 
   /* reset the new_files list */
-  thunar_file_list_free (folder->new_files);
+  thunar_g_file_list_free (folder->new_files);
   folder->new_files = NULL;
 
   /* start a new job */
