@@ -23,8 +23,10 @@
 #endif
 
 #include <gio/gio.h>
+
 #ifdef HAVE_GIO_UNIX
 #include <gio/gunixmounts.h>
+#include <gio/gdesktopappinfo.h>
 #endif
 
 #include <exo/exo.h>
@@ -255,6 +257,75 @@ thunar_g_file_get_display_name (GFile *file)
 
 
 
+gchar *
+thunar_g_file_get_display_name_remote (GFile *mount_point)
+{
+  gchar       *scheme;
+  gchar       *parse_name;
+  const gchar *p;
+  const gchar *path;
+  gchar       *hostname;
+  gchar       *display_name = NULL;
+  const gchar *skip;
+  const gchar  skip_chars[] = G_URI_RESERVED_CHARS_ALLOWED_IN_PATH_ELEMENT;
+  guint        n;
+
+  _thunar_return_val_if_fail (G_IS_FILE (mount_point), NULL);
+
+  /* not intended for local mounts */
+  if (!g_file_is_native (mount_point))
+    {
+      scheme = g_file_get_uri_scheme (mount_point);
+      parse_name = g_file_get_parse_name (mount_point);
+
+      if (g_str_has_prefix (parse_name, scheme))
+        {
+          /* extract the hostname */
+          p = parse_name + strlen (scheme);
+          while (*p == ':' || *p == '/')
+            ++p;
+
+          /* goto path part */
+          path = strchr (p, '/');
+
+          /* skip password or login names in the hostname */
+          for (n = 0; n < G_N_ELEMENTS (skip_chars) - 1; n++)
+            {
+              skip = strchr (p, skip_chars[n]);
+              if (skip != NULL && (path == NULL || skip < path))
+                p = skip + 1;
+            }
+
+          /* extract the path and hostname from the string */
+          if (G_LIKELY (path != NULL))
+            {
+              hostname = g_strndup (p, path - p);
+            }
+          else
+            {
+              hostname = g_strdup (p);
+              path = "/";
+            }
+
+          /* TRANSLATORS: this will result in "<path> on <hostname>" */
+          display_name = g_strdup_printf (_("%s on %s"), path, hostname);
+
+          g_free (hostname);
+        }
+
+      g_free (scheme);
+      g_free (parse_name);
+    }
+
+  /* never return null */
+  if (display_name == NULL)
+    display_name = thunar_g_file_get_display_name (mount_point);
+
+  return display_name;
+}
+
+
+
 gboolean
 thunar_g_vfs_is_uri_scheme_supported (const gchar *scheme)
 {
@@ -357,66 +428,6 @@ thunar_g_file_list_to_stringv (GList *list)
 
 
 
-GList *
-thunar_g_file_list_append (GList *list,
-                           GFile *file)
-{
-  return g_list_append (list, g_object_ref (file));
-}
-
-
-
-GList *
-thunar_g_file_list_prepend (GList *list,
-                            GFile *file)
-{
-  return g_list_prepend (list, g_object_ref (file));
-}
-
-
-
-/**
- * thunar_g_file_list_copy:
- * @list : a list of #GFile<!---->s.
- *
- * Takes a deep copy of @list and returns the
- * result. The caller is responsible to free the
- * returned list using thunar_g_file_list_free().
- *
- * Return value: a deep copy of @list.
- **/
-GList*
-thunar_g_file_list_copy (GList *list)
-{
-  GList *copy = NULL;
-  GList *lp;
-
-  for (lp = g_list_last (list); lp != NULL; lp = lp->prev)
-    copy = g_list_prepend (copy, g_object_ref (lp->data));
-
-  return copy;
-}
-
-
-
-/**
- * thunar_g_file_list_free:
- * @list : a list of #GFile<!---->s.
- *
- * Frees the #GFile<!---->s in @list and
- * the @list itself.
- **/
-void
-thunar_g_file_list_free (GList *list)
-{
-  GList *lp;
-  for (lp = list; lp != NULL; lp = lp->next)
-    g_object_unref (lp->data);
-  g_list_free (list);
-}
-
-
-
 gboolean
 thunar_g_app_info_launch (GAppInfo          *info,
                           GFile             *working_directory,
@@ -467,3 +478,34 @@ thunar_g_app_info_launch (GAppInfo          *info,
   return result;
 }
 
+
+
+gboolean
+thunar_g_app_info_should_show (GAppInfo *info)
+{
+#ifdef HAVE_GIO_UNIX
+  const gchar *filename;
+
+  _thunar_return_val_if_fail (G_IS_APP_INFO (info), FALSE);
+
+  /* check if NoDesktop is set or OnlyShowIn is set but
+   * does not contain XFCE */
+  if (g_app_info_should_show (info))
+    return TRUE;
+
+  if (G_IS_DESKTOP_APP_INFO (info))
+    {
+      /* show custom command from the user directory, this to not
+       * exclude custom commands */
+      filename = g_desktop_app_info_get_filename (G_DESKTOP_APP_INFO (info));
+      if (filename != NULL
+          && g_str_has_prefix (filename, g_get_user_data_dir ()))
+        return TRUE;
+    }
+
+  return FALSE;
+#else
+  /* we cannot exclude custom actions, so show everything */
+  return TRUE;
+#endif
+}
