@@ -38,11 +38,8 @@
 
 
 
-/* the timeout until the sweeper is run (in ms) */
-#define THUNAR_ICON_FACTORY_SWEEP_TIMEOUT (30 * 1000)
-
-/* the maximum length of the recently used list */
-#define MAX_RECENTLY (32u)
+/* the timeout until the sweeper is run (in seconds) */
+#define THUNAR_ICON_FACTORY_SWEEP_TIMEOUT (30)
 
 
 
@@ -83,11 +80,10 @@ static GdkPixbuf *thunar_icon_factory_lookup_icon           (ThunarIconFactory
                                                              const gchar              *name,
                                                              gint                      size,
                                                              gboolean                  wants_default);
-static void       thunar_icon_factory_mark_recently_used    (ThunarIconFactory        *factory,
-                                                             GdkPixbuf                *pixbuf);
 static guint      thunar_icon_key_hash                      (gconstpointer             data);
 static gboolean   thunar_icon_key_equal                     (gconstpointer             a,
                                                              gconstpointer             b);
+static void       thunar_icon_key_free                      (gpointer                  data);
 static GdkPixbuf *thunar_icon_factory_load_fallback         (ThunarIconFactory        *factory,
                                                              gint                      size);
 
@@ -104,19 +100,18 @@ struct _ThunarIconFactory
 
   ThunarPreferences *preferences;
 
-  GdkPixbuf         *recently[MAX_RECENTLY];  /* ring buffer */
-  guint              recently_pos;            /* insert position */
-
   GHashTable        *icon_cache;
 
   GtkIconTheme      *icon_theme;
 
-  gboolean           show_thumbnails;
+  guint              show_thumbnails : 1;
 
-  gint               changed_idle_id;
-  gint               sweep_timer_id;
+  guint              sweep_timer_id;
 
   gulong             changed_hook_id;
+
+  /* stamp that gets bumped when the theme changes */
+  guint              theme_stamp;
 };
 
 struct _ThunarIconKey
@@ -125,12 +120,20 @@ struct _ThunarIconKey
   gint   size;
 };
 
+typedef struct
+{
+  ThunarFileIconState   icon_state;
+  ThunarFileThumbState  thumb_state;
+  gint                  icon_size;
+  guint                 stamp;
+  GdkPixbuf            *icon;
+}
+ThunarIconStore;
+
 
 
 static GQuark thunar_icon_factory_quark = 0;
-static GQuark thunar_icon_thumb_uri_quark = 0;
-static GQuark thunar_icon_thumb_time_quark = 0;
-static GQuark thunar_file_thumb_uri_quark = 0;
+static GQuark thunar_icon_factory_store_quark = 0;
 
 
 
@@ -143,12 +146,7 @@ thunar_icon_factory_class_init (ThunarIconFactoryClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* setup the thunar-icon-thumb-{path,time} quarks */
-  thunar_icon_thumb_uri_quark = g_quark_from_static_string ("thunar-icon-thumb-path");
-  thunar_icon_thumb_time_quark = g_quark_from_static_string ("thunar-icon-thumb-time");
-
-  /* setup the thunar-file-thumb-path quark */
-  thunar_file_thumb_uri_quark = g_quark_from_static_string ("thunar-file-thumb-path");
+  thunar_icon_factory_store_quark = g_quark_from_static_string ("thunar-icon-factory-store");
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_icon_factory_dispose;
@@ -190,10 +188,6 @@ thunar_icon_factory_class_init (ThunarIconFactoryClass *klass)
 static void
 thunar_icon_factory_init (ThunarIconFactory *factory)
 {
-  /* initialize GSource ids */
-  factory->changed_idle_id = -1;
-  factory->sweep_timer_id = -1;
-
   /* connect emission hook for the "changed" signal on the GtkIconTheme class. We use the emission
    * hook way here, because that way we can make sure that the icon cache is definetly cleared
    * before any other part of the application gets notified about the icon theme change.
@@ -202,7 +196,8 @@ thunar_icon_factory_init (ThunarIconFactory *factory)
                                                          0, thunar_icon_factory_changed, factory, NULL);
 
   /* allocate the hash table for the icon cache */
-  factory->icon_cache = g_hash_table_new_full (thunar_icon_key_hash, thunar_icon_key_equal, g_free, g_object_unref);
+  factory->icon_cache = g_hash_table_new_full (thunar_icon_key_hash, thunar_icon_key_equal,
+                                               thunar_icon_key_free, g_object_unref);
 }
 
 
@@ -214,10 +209,7 @@ thunar_icon_factory_dispose (GObject *object)
 
   _thunar_return_if_fail (THUNAR_IS_ICON_FACTORY (factory));
 
-  if (G_UNLIKELY (factory->changed_idle_id >= 0))
-    g_source_remove (factory->changed_idle_id);
-
-  if (G_UNLIKELY (factory->sweep_timer_id >= 0))
+  if (G_UNLIKELY (factory->sweep_timer_id != 0))
     g_source_remove (factory->sweep_timer_id);
 
   (*G_OBJECT_CLASS (thunar_icon_factory_parent_class)->dispose) (object);
@@ -229,15 +221,9 @@ static void
 thunar_icon_factory_finalize (GObject *object)
 {
   ThunarIconFactory *factory = THUNAR_ICON_FACTORY (object);
-  guint              n;
 
   _thunar_return_if_fail (THUNAR_IS_ICON_FACTORY (factory));
 
-  /* clear the recently used list */
-  for (n = 0; n < MAX_RECENTLY; ++n)
-    if (G_LIKELY (factory->recently[n] != NULL))
-      g_object_unref (G_OBJECT (factory->recently[n]));
-
   /* clear the icon cache hash table */
   g_hash_table_destroy (factory->icon_cache);
 
@@ -314,18 +300,12 @@ thunar_icon_factory_changed (GSignalInvocationHint *ihint,
                              gpointer               user_data)
 {
   ThunarIconFactory *factory = THUNAR_ICON_FACTORY (user_data);
-  guint              n;
-
-  /* drop all items from the recently used list */
-  for (n = 0; n < MAX_RECENTLY; ++n)
-    if (G_LIKELY (factory->recently[n] != NULL))
-      {
-        g_object_unref (G_OBJECT (factory->recently[n]));
-        factory->recently[n] = NULL;
-      }
 
   /* drop all items from the icon cache */
-  g_hash_table_foreach_remove (factory->icon_cache, (GHRFunc) exo_noop_true, NULL);
+  g_hash_table_remove_all (factory->icon_cache);
+
+  /* bump the stamp so all file icons are reloaded */
+  factory->theme_stamp++;
 
   /* keep the emission hook alive */
   return TRUE;
@@ -364,7 +344,7 @@ thunar_icon_factory_sweep_timer (gpointer user_data)
 static void
 thunar_icon_factory_sweep_timer_destroy (gpointer user_data)
 {
-  THUNAR_ICON_FACTORY (user_data)->sweep_timer_id = -1;
+  THUNAR_ICON_FACTORY (user_data)->sweep_timer_id = 0;
 }
 
 
@@ -469,7 +449,7 @@ thunar_icon_factory_load_from_file (ThunarIconFactory *factory,
         {
           /* add a frame to the thumbnail */
           frame = gdk_pixbuf_new_from_inline (-1, thunar_thumbnail_frame, FALSE, NULL);
-          tmp = exo_gdk_pixbuf_frame (pixbuf, frame, 3, 3, 6, 6);
+          tmp = exo_gdk_pixbuf_frame (pixbuf, frame, 4, 3, 5, 6);
           g_object_unref (G_OBJECT (pixbuf));
           g_object_unref (G_OBJECT (frame));
           pixbuf = tmp;
@@ -534,61 +514,27 @@ thunar_icon_factory_lookup_icon (ThunarIconFactory *factory,
         }
 
       /* generate a key for the new cached icon */
-      key = g_malloc (sizeof (ThunarIconKey) + strlen (name) + 1);
-      key->name = ((gchar *) key) + sizeof (ThunarIconKey);
+      key = g_slice_new (ThunarIconKey);
       key->size = size;
-      strcpy (key->name, name);
+      key->name = g_strdup (name);
 
       /* insert the new icon into the cache */
       g_hash_table_insert (factory->icon_cache, key, pixbuf);
     }
 
-  /* add the icon to the recently used list */
-  thunar_icon_factory_mark_recently_used (factory, pixbuf);
+  /* schedule the sweeper */
+  if (G_UNLIKELY (factory->sweep_timer_id == 0))
+    {
+      factory->sweep_timer_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, THUNAR_ICON_FACTORY_SWEEP_TIMEOUT,
+                                                            thunar_icon_factory_sweep_timer, factory,
+                                                            thunar_icon_factory_sweep_timer_destroy);
+    }
 
   return g_object_ref (G_OBJECT (pixbuf));
 }
 
 
 
-static void
-thunar_icon_factory_mark_recently_used (ThunarIconFactory *factory,
-                                        GdkPixbuf         *pixbuf)
-{
-  guint n;
-
-  _thunar_return_if_fail (THUNAR_IS_ICON_FACTORY (factory));
-  _thunar_return_if_fail (GDK_IS_PIXBUF (pixbuf));
-
-  /* check if the icon is already on the list */
-  for (n = 0; n < MAX_RECENTLY; ++n)
-    if (G_UNLIKELY (factory->recently[n] == pixbuf))
-      return;
-
-  /* ditch the previous item on the current insert position,
-   * which - if present - is the oldest item in the list.
-   */
-  if (G_LIKELY (factory->recently[factory->recently_pos] != NULL))
-    g_object_unref (G_OBJECT (factory->recently[factory->recently_pos]));
-
-  /* insert the new pixbuf into the list */
-  factory->recently[factory->recently_pos] = pixbuf;
-  g_object_ref (G_OBJECT (pixbuf));
-
-  /* advance the insert position */
-  factory->recently_pos = (factory->recently_pos + 1) % MAX_RECENTLY;
-
-  /* schedule the sweeper */
-  if (G_UNLIKELY (factory->sweep_timer_id < 0))
-    {
-      factory->sweep_timer_id = g_timeout_add_full (G_PRIORITY_LOW, THUNAR_ICON_FACTORY_SWEEP_TIMEOUT,
-                                                    thunar_icon_factory_sweep_timer, factory,
-                                                    thunar_icon_factory_sweep_timer_destroy);
-    }
-}
-
-
-
 static guint
 thunar_icon_key_hash (gconstpointer data)
 {
@@ -618,7 +564,30 @@ thunar_icon_key_equal (gconstpointer a,
     return FALSE;
 
   /* do a full string comparison on the names */
-  return (strcmp (a_key->name, b_key->name) == 0);
+  return exo_str_is_equal (a_key->name, b_key->name);
+}
+
+
+
+static void
+thunar_icon_key_free (gpointer data)
+{
+  ThunarIconKey *key = data;
+
+  g_free (key->name);
+  g_slice_free (ThunarIconKey, key);
+}
+
+
+
+static void
+thunar_icon_store_free (gpointer data)
+{
+  ThunarIconStore *store = data;
+
+  if (store->icon != NULL)
+    g_object_unref (store->icon);
+  g_slice_free (ThunarIconStore, store);
 }
 
 
@@ -716,7 +685,6 @@ thunar_icon_factory_get_for_icon_theme (GtkIconTheme *icon_theme)
  * @factory       : a #ThunarIconFactory instance.
  * @name          : name of the icon to load.
  * @size          : desired icon size.
- * @attach_points : location to store the attach points to or %NULL.
  * @wants_default : %TRUE to return the fallback icon if no icon of @name
  *                  is found in the @factory.
  *
@@ -734,7 +702,6 @@ GdkPixbuf*
 thunar_icon_factory_load_icon (ThunarIconFactory        *factory,
                                const gchar              *name,
                                gint                      size,
-                               ThunarEmblemAttachPoints *attach_points,
                                gboolean                  wants_default)
 {
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
@@ -743,7 +710,7 @@ thunar_icon_factory_load_icon (ThunarIconFactory        *factory,
   /* cannot happen unless there's no XSETTINGS manager
    * for the default screen, but just in case...
    */
-  if (G_UNLIKELY (name == NULL || *name == '\0'))
+  if (G_UNLIKELY (exo_str_is_empty (name)))
     {
       /* check if the caller will happly accept the fallback icon */
       if (G_LIKELY (wants_default))
@@ -776,24 +743,36 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
                                     ThunarFileIconState icon_state,
                                     gint                icon_size)
 {
-  GInputStream        *stream;
-  GtkIconInfo         *icon_info;
-  const gchar         *thumbnail_path;
-  GdkPixbuf           *icon = NULL;
-  GIcon               *gicon;
-  gchar               *icon_name;
+  GInputStream    *stream;
+  GtkIconInfo     *icon_info;
+  const gchar     *thumbnail_path;
+  GdkPixbuf       *icon = NULL;
+  GIcon           *gicon;
+  gchar           *icon_name;
+  const gchar     *custom_icon;
+  ThunarIconStore *store;
 
   _thunar_return_val_if_fail (THUNAR_IS_ICON_FACTORY (factory), NULL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
   _thunar_return_val_if_fail (icon_size > 0, NULL);
 
+  /* check if we have a stored icon on the file and it is still valid */
+  store = g_object_get_qdata (G_OBJECT (file), thunar_icon_factory_store_quark);
+  if (store != NULL
+      && store->icon_state == icon_state
+      && store->icon_size == icon_size
+      && store->stamp == factory->theme_stamp
+      && store->thumb_state == thunar_file_get_thumb_state (file))
+    {
+      return g_object_ref (store->icon);
+    }
+
   /* check if we have a custom icon for this file */
-  icon_name = thunar_file_get_custom_icon (file);
-  if (icon_name != NULL)
+  custom_icon = thunar_file_get_custom_icon (file);
+  if (custom_icon != NULL)
     {
       /* try to load the icon */
-      icon = thunar_icon_factory_lookup_icon (factory, icon_name, icon_size, FALSE);
-      g_free (icon_name);
+      icon = thunar_icon_factory_lookup_icon (factory, custom_icon, icon_size, FALSE);
       if (G_LIKELY (icon != NULL))
         return icon;
     }
@@ -813,7 +792,8 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
               icon_info = 
                 gtk_icon_theme_lookup_by_gicon (factory->icon_theme, 
                                                 gicon, icon_size, 
-                                                GTK_ICON_LOOKUP_USE_BUILTIN);
+                                                GTK_ICON_LOOKUP_USE_BUILTIN
+                                                | GTK_ICON_LOOKUP_FORCE_SIZE);
 
               /* check if the lookup succeeded */
               if (icon_info != NULL)
@@ -833,7 +813,9 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
               if (stream != NULL)
                 {
                   /* load the pixbuf from the stream */
-                  icon = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+                  icon = gdk_pixbuf_new_from_stream_at_scale (stream, icon_size,
+                                                              icon_size, TRUE,
+                                                              NULL, NULL);
 
                   /* destroy the stream */
                   g_object_unref (stream);
@@ -854,23 +836,31 @@ thunar_icon_factory_load_file_icon (ThunarIconFactory  *factory,
           if (thumbnail_path != NULL)
             {
               /* try to load the thumbnail */
-              icon = thunar_icon_factory_load_from_file (factory, thumbnail_path, 
-                                                         icon_size);
-
-              /* return the thumbnail if it could be loaded */
-              if (icon != NULL)
-                return icon;
+              icon = thunar_icon_factory_load_from_file (factory, thumbnail_path, icon_size);
             }
         }
     }
 
   /* lookup the icon name for the icon in the given state and load the icon */
-  icon_name = thunar_file_get_icon_name (file, icon_state, factory->icon_theme);
-  icon = thunar_icon_factory_load_icon (factory, icon_name, icon_size, NULL, TRUE);
-  g_free (icon_name);
+  if (G_LIKELY (icon == NULL))
+    {
+      icon_name = thunar_file_get_icon_name (file, icon_state, factory->icon_theme);
+      icon = thunar_icon_factory_load_icon (factory, icon_name, icon_size, TRUE);
+      g_free (icon_name);
+    }
+
+  if (G_LIKELY (icon != NULL))
+    {
+      store = g_slice_new (ThunarIconStore);
+      store->icon_size = icon_size;
+      store->icon_state = icon_state;
+      store->stamp = factory->theme_stamp;
+      store->thumb_state = thunar_file_get_thumb_state (file);
+      store->icon = g_object_ref (icon);
+
+      g_object_set_qdata_full (G_OBJECT (file), thunar_icon_factory_store_quark,
+                               store, thunar_icon_store_free);
+    }
+
   return icon;
 }
-
-
-
-
