@@ -1,7 +1,7 @@
 /* $Id$ */
 /*-
  * Copyright (c) 2004-2007 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -164,9 +164,6 @@ static gint               sort_by_date_accessed                   (const ThunarF
 static gint               sort_by_date_modified                   (const ThunarFile       *a,
                                                                    const ThunarFile       *b,
                                                                    gboolean                case_sensitive);
-static gint               sort_by_file_name                       (const ThunarFile       *a,
-                                                                   const ThunarFile       *b,
-                                                                   gboolean                case_sensitive);
 static gint               sort_by_group                           (const ThunarFile       *a,
                                                                    const ThunarFile       *b,
                                                                    gboolean                case_sensitive);
@@ -189,6 +186,14 @@ static gint               sort_by_type                            (const ThunarF
                                                                    const ThunarFile       *b,
                                                                    gboolean                case_sensitive);
 
+static gboolean           thunar_list_model_get_case_sensitive    (ThunarListModel        *store);
+static void               thunar_list_model_set_case_sensitive    (ThunarListModel        *store,
+                                                                   gboolean                case_sensitive);
+static ThunarDateStyle    thunar_list_model_get_date_style        (ThunarListModel        *store);
+static void               thunar_list_model_set_date_style        (ThunarListModel        *store,
+                                                                   ThunarDateStyle         date_style);
+static gint               thunar_list_model_get_num_files         (ThunarListModel        *store);
+static gboolean           thunar_list_model_get_folders_first     (ThunarListModel        *store);
 
 
 struct _ThunarListModelClass
@@ -870,7 +875,7 @@ thunar_list_model_get_sort_column_id (GtkTreeSortable *sortable,
     *sort_column_id = THUNAR_COLUMN_MIME_TYPE;
   else if (store->sort_func == sort_by_name)
     *sort_column_id = THUNAR_COLUMN_NAME;
-  else if (store->sort_func == sort_by_file_name)
+  else if (store->sort_func == sort_by_name)
     *sort_column_id = THUNAR_COLUMN_FILE_NAME;
   else if (store->sort_func == sort_by_permissions)
     *sort_column_id = THUNAR_COLUMN_PERMISSIONS;
@@ -929,6 +934,7 @@ thunar_list_model_set_sort_column_id (GtkTreeSortable *sortable,
       store->sort_func = sort_by_mime_type;
       break;
 
+    case THUNAR_COLUMN_FILE_NAME:
     case THUNAR_COLUMN_NAME:
       store->sort_func = sort_by_name;
       break;
@@ -949,10 +955,6 @@ thunar_list_model_set_sort_column_id (GtkTreeSortable *sortable,
       store->sort_func = sort_by_type;
       break;
 
-    case THUNAR_COLUMN_FILE_NAME:
-      store->sort_func = sort_by_file_name;
-      break;
-
     default:
       _thunar_assert_not_reached ();
     }
@@ -1011,11 +1013,8 @@ thunar_list_model_cmp (ThunarListModel *store,
     {
       isdir_a = thunar_file_is_directory (a);
       isdir_b = thunar_file_is_directory (b);
-
-      if (isdir_a && !isdir_b)
-        return -1;
-      else if (!isdir_a && isdir_b)
-        return 1;
+      if (isdir_a != isdir_b)
+        return isdir_a ? -1 : 1;
     }
 
   return (*store->sort_func) (a, b, store->sort_case_sensitive) * store->sort_sign;
@@ -1402,22 +1401,6 @@ sort_by_date_modified (const ThunarFile *a,
 
 
 
-static gint
-sort_by_file_name (const ThunarFile *a,
-                   const ThunarFile *b,
-                   gboolean          case_sensitive)
-{
-  const gchar *a_name = thunar_file_get_display_name (a);
-  const gchar *b_name = thunar_file_get_display_name (b);
-
-  if (!case_sensitive)
-    return strcasecmp (a_name, b_name);
-  else
-    return strcmp (a_name, b_name);
-}
-
-
-
 static gint
 sort_by_group (const ThunarFile *a,
                const ThunarFile *b,
@@ -1484,6 +1467,11 @@ sort_by_mime_type (const ThunarFile *a,
   content_type_a = thunar_file_get_content_type (a);
   content_type_b = thunar_file_get_content_type (b);
 
+  if (content_type_a == NULL)
+    content_type_a = "";
+  if (content_type_b == NULL)
+    content_type_b = "";
+
   result = strcasecmp (content_type_a, content_type_b);
 
   if (result == 0)
@@ -1643,6 +1631,15 @@ sort_by_type (const ThunarFile *a,
       description_b = g_content_type_get_description (content_type_b);
     }
 
+  /* avoid calling strcasecmp with NULL parameters */
+  if (description_a == NULL || description_b == NULL)
+    {
+      g_free (description_a);
+      g_free (description_b);
+
+      return 0;
+    }
+
   if (!case_sensitive)
     result = strcasecmp (description_a, description_b);
   else
@@ -1675,24 +1672,6 @@ thunar_list_model_new (void)
 
 
 
-/**
- * thunar_list_model_new_with_folder:
- * @folder : a valid #ThunarFolder object.
- *
- * Allocates a new #ThunarListModel instance and associates
- * it with @folder.
- *
- * Return value: the newly created #ThunarListModel instance.
- **/
-ThunarListModel*
-thunar_list_model_new_with_folder (ThunarFolder *folder)
-{
-  _thunar_return_val_if_fail (THUNAR_IS_FOLDER (folder), NULL);
-  return g_object_new (THUNAR_TYPE_LIST_MODEL, "folder", folder, NULL);
-}
-
-
-
 /**
  * thunar_list_model_get_case_sensitive:
  * @store : a valid #ThunarListModel object.
@@ -1702,7 +1681,7 @@ thunar_list_model_new_with_folder (ThunarFolder *folder)
  *
  * Return value: %TRUE if sorting is case-sensitive.
  **/
-gboolean
+static gboolean
 thunar_list_model_get_case_sensitive (ThunarListModel *store)
 {
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), FALSE);
@@ -1719,7 +1698,7 @@ thunar_list_model_get_case_sensitive (ThunarListModel *store)
  * If @case_sensitive is %TRUE the sorting in @store will be done
  * in a case-sensitive manner.
  **/
-void
+static void
 thunar_list_model_set_case_sensitive (ThunarListModel *store,
                                       gboolean         case_sensitive)
 {
@@ -1751,7 +1730,7 @@ thunar_list_model_set_case_sensitive (ThunarListModel *store,
  * Return value: the #ThunarDateStyle used to format dates in
  *               the given @store.
  **/
-ThunarDateStyle
+static ThunarDateStyle
 thunar_list_model_get_date_style (ThunarListModel *store)
 {
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), THUNAR_DATE_STYLE_SIMPLE);
@@ -1769,7 +1748,7 @@ thunar_list_model_get_date_style (ThunarListModel *store)
  * Chances the style used to format dates in @store to the specified
  * @date_style.
  **/
-void
+static void
 thunar_list_model_set_date_style (ThunarListModel *store,
                                   ThunarDateStyle  date_style)
 {
@@ -1867,8 +1846,7 @@ thunar_list_model_set_folder (ThunarListModel *store,
       g_slist_free (rows);
 
       /* remove hidden entries */
-      g_slist_foreach (store->hidden, (GFunc) g_object_unref, NULL);
-      g_slist_free (store->hidden);
+      g_slist_free_full (store->hidden, g_object_unref);
       store->hidden = NULL;
 
       /* unregister signals and drop the reference */
@@ -1963,7 +1941,7 @@ thunar_list_model_set_folder (ThunarListModel *store,
  *
  * Return value: %TRUE if @store lists folders first.
  **/
-gboolean
+static gboolean
 thunar_list_model_get_folders_first (ThunarListModel *store)
 {
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), FALSE);
@@ -2139,7 +2117,7 @@ thunar_list_model_get_file (ThunarListModel *store,
  *
  * Return value: ahe number of visible files in @store.
  **/
-gint
+static gint
 thunar_list_model_get_num_files (ThunarListModel *store)
 {
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), 0);
@@ -2161,8 +2139,7 @@ thunar_list_model_get_num_files (ThunarListModel *store)
  *
  * The caller is responsible to free the returned list using:
  * <informalexample><programlisting>
- * g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
- * g_list_free (list);
+ * g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
  * </programlisting></informalexample>
  *
  * Return value: the list of #GtkTreePath<!---->s for @files.
@@ -2197,8 +2174,7 @@ thunar_list_model_get_paths_for_files (ThunarListModel *store,
  *
  * The caller is responsible to free the returned list using:
  * <informalexample><programlisting>
- * g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
- * g_list_free (list);
+ * g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
  * </programlisting></informalexample>
  *
  * Return value: the list of #GtkTreePath<!---->s that match @pattern.
@@ -2260,6 +2236,8 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
   ThunarFile  *file;
   guint64      size;
   guint64      size_summary;
+  gint         folder_count;
+  gint         non_folder_count;
   GSList      *row;
   GList       *lp;
   gchar       *absolute_path;
@@ -2267,10 +2245,12 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
   gchar       *display_name;
   gchar       *size_string;
   gchar       *text;
+  gchar       *folder_text;
+  gchar       *non_folder_text;
   gchar       *s;
   gint         height;
   gint         width;
-  gint         n;
+  gchar       *description;
 
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), NULL);
 
@@ -2280,32 +2260,34 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
       file = (store->folder != NULL) ? thunar_folder_get_corresponding_file (store->folder) : NULL;
 
       /* check if we can determine the amount of free space for the volume */
-      if (G_LIKELY (file != NULL && thunar_file_get_free_space (file, &size)))
+      if (G_LIKELY (file != NULL && thunar_file_get_free_space (file, &size, NULL)))
         {
           /* humanize the free space */
-          fspace_string = g_format_size_for_display (size);
+          fspace_string = g_format_size (size);
+          size_summary = 0;
 
           /* check if we have atleast one file in this folder */
           if (G_LIKELY (store->nrows > 0))
             {
-              /* calculate the size of all items */
-              for (row = store->rows, size_summary = 0; row != NULL; row = row->next)
-                size_summary += thunar_file_get_size (row->data);
-
-              /* humanize the size summary */
-              size_string = g_format_size_for_display (size_summary);
+              /* calculate the size of all file items */
+              for (row = store->rows; row != NULL; row = row->next)
+                if (thunar_file_is_regular (row->data))
+                  size_summary += thunar_file_get_size (row->data);
+            }
 
+          if (size_summary > 0)
+            {
               /* generate a text which includes the size of all items in the folder */
+              size_string = g_format_size (size_summary);
               text = g_strdup_printf (ngettext ("%d item (%s), Free space: %s", "%d items (%s), Free space: %s", store->nrows),
                                       store->nrows, size_string, fspace_string);
-
-              /* cleanup */
               g_free (size_string);
             }
           else
             {
               /* just the standard text */
-              text = g_strdup_printf (ngettext ("%d item, Free space: %s", "%d items, Free space: %s", store->nrows), store->nrows, fspace_string);
+              text = g_strdup_printf (ngettext ("%d item, Free space: %s", "%d items, Free space: %s", store->nrows),
+                                      store->nrows, fspace_string);
             }
 
           /* cleanup */
@@ -2323,21 +2305,20 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
 
       /* get the file for the given iter */
       file = THUNAR_FILE (G_SLIST (iter.user_data)->data);
-      
+
       /* determine the content type of the file */
       content_type = thunar_file_get_content_type (file);
 
-      /* calculate the text to be displayed */
-      size_string = thunar_file_get_size_string (file);
-
       if (G_UNLIKELY (content_type != NULL && g_str_equal (content_type, "inode/symlink")))
         {
           text = g_strdup_printf (_("\"%s\" broken link"), thunar_file_get_display_name (file));
         }
       else if (G_UNLIKELY (thunar_file_is_symlink (file)))
         {
+          size_string = thunar_file_get_size_string (file);
           text = g_strdup_printf (_("\"%s\" (%s) link to %s"), thunar_file_get_display_name (file),
                                   size_string, thunar_file_get_symlink_target (file));
+          g_free (size_string);
         }
       else if (G_UNLIKELY (thunar_file_get_kind (file) == G_FILE_TYPE_SHORTCUT))
         {
@@ -2347,16 +2328,21 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
         {
           text = g_strdup_printf (_("\"%s\" mountable"), thunar_file_get_display_name (file));
         }
-      else
+      else if (thunar_file_is_regular (file))
         {
-          gchar *description;
-          
-          description = g_content_type_get_description (thunar_file_get_content_type (file));
+          description = g_content_type_get_description (content_type);
+          size_string = thunar_file_get_size_string (file);
           text = g_strdup_printf (_("\"%s\" (%s) %s"), thunar_file_get_display_name (file),
                                   size_string, description);
           g_free (description);
+          g_free (size_string);
+        }
+      else
+        {
+          description = g_content_type_get_description (content_type);
+          text = g_strdup_printf (_("\"%s\" %s"), thunar_file_get_display_name (file), description);
+          g_free (description);
         }
-      g_free (size_string);
 
       /* append the original path (if any) */
       original_path = thunar_file_get_original_path (file);
@@ -2369,7 +2355,9 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
           g_free (text);
           text = s;
         }
-      else if (thunar_file_is_local (file) && thunar_file_is_regular (file))
+      else if (thunar_file_is_local (file)
+               && thunar_file_is_regular (file)
+               && g_str_has_prefix (content_type, "image/")) /* bug #2913 */
         {
           /* check if we can determine the dimension of this file (only for image files) */
           absolute_path = g_file_get_path (thunar_file_get_file (file));
@@ -2386,22 +2374,78 @@ thunar_list_model_get_statusbar_text (ThunarListModel *store,
     }
   else
     {
-      /* sum up all sizes */
-      for (lp = selected_items, n = 0, size_summary = 0; lp != NULL; lp = lp->next, ++n)
+      /* reset */
+      size_summary = 0;
+      folder_count = 0;
+      non_folder_count = 0;
+
+      /* analyze selection */
+      for (lp = selected_items; lp != NULL; lp = lp->next)
         {
           gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, lp->data);
-          size_summary += thunar_file_get_size (THUNAR_FILE (G_SLIST (iter.user_data)->data));
+          file = THUNAR_FILE (G_SLIST (iter.user_data)->data);
+          if (thunar_file_is_directory (file))
+            {
+              folder_count++;
+            }
+          else
+            {
+              non_folder_count++;
+              if (thunar_file_is_regular (file))
+                size_summary += thunar_file_get_size (file);
+            }
         }
 
-      if (size_summary > 0)
+     /* text for the items in the folder */
+     if (non_folder_count > 0)
         {
-          size_string = g_format_size_for_display (size_summary);
-          text = g_strdup_printf (ngettext ("%d item selected (%s)", "%d items selected (%s)", n), n, size_string);
+          size_string = g_format_size (size_summary);
+          if (folder_count > 0)
+            {
+              /* item count if there are also folders in the selection */
+              non_folder_text = g_strdup_printf (ngettext ("%d other item selected (%s)",
+                                                           "%d other items selected (%s)",
+                                                           non_folder_count), non_folder_count, size_string);
+            }
+          else
+            {
+              /* only non-folders are selected */
+              non_folder_text = g_strdup_printf (ngettext ("%d item selected (%s)",
+                                                           "%d items selected (%s)",
+                                                           non_folder_count), non_folder_count, size_string);
+            }
           g_free (size_string);
         }
       else
         {
-          text = g_strdup_printf (ngettext ("%d item selected", "%d items selected", n), n);
+          non_folder_text = NULL;
+        }
+
+      /* text for the folders */
+      if (folder_count > 0)
+        {
+          folder_text = g_strdup_printf (ngettext ("%d folder selected",
+                                                   "%d folders selected",
+                                                   folder_count), folder_count);
+        }
+      else
+        {
+          folder_text = NULL;
+        }
+
+      if (folder_text == NULL)
+        text = non_folder_text;
+      else if (non_folder_text == NULL)
+        text = folder_text;
+      else
+        {
+          /* This is marked for translation in case a localizer
+           * needs to change ", " to something else. The comma
+           * is between the message about the number of folders
+           * and the number of items in the selection */
+          text = g_strdup_printf (_("%s, %s"), folder_text, non_folder_text);
+          g_free (folder_text);
+          g_free (non_folder_text);
         }
     }
 
