@@ -64,73 +64,80 @@ enum
 
 
 
-static void     thunar_renamer_dialog_dispose             (GObject                  *object);
-static void     thunar_renamer_dialog_finalize            (GObject                  *object);
-static void     thunar_renamer_dialog_get_property        (GObject                  *object,
-                                                           guint                     prop_id,
-                                                           GValue                   *value,
-                                                           GParamSpec               *pspec);
-static void     thunar_renamer_dialog_set_property        (GObject                  *object,
-                                                           guint                     prop_id,
-                                                           const GValue             *value,
-                                                           GParamSpec               *pspec);
-static void     thunar_renamer_dialog_realize             (GtkWidget                *widget);
-static void     thunar_renamer_dialog_unrealize           (GtkWidget                *widget);
-static void     thunar_renamer_dialog_response            (GtkDialog                *dialog,
-                                                           gint                      response);
-static void     thunar_renamer_dialog_context_menu        (ThunarRenamerDialog      *renamer_dialog,
-                                                           guint                     button,
-                                                           guint32                   timestamp);
-static void     thunar_renamer_dialog_help                (ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_save                (ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_action_add_files    (GtkAction                *action,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_action_remove_files (GtkAction                *action,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_action_clear        (GtkAction                *action,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_action_about        (GtkAction                *action,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_action_properties   (GtkAction                *action,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static gboolean thunar_renamer_dialog_button_press_event  (GtkWidget                *tree_view,
-                                                           GdkEventButton           *event,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_drag_data_received  (GtkWidget                *tree_view,
-                                                           GdkDragContext           *context,
-                                                           gint                      x,
-                                                           gint                      y,
-                                                           GtkSelectionData         *selection_data,
-                                                           guint                     info,
-                                                           guint                     timestamp,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_drag_leave          (GtkWidget                *tree_view,
-                                                           GdkDragContext           *context,
-                                                           guint                     timestamp,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static gboolean thunar_renamer_dialog_drag_motion         (GtkWidget                *tree_view,
-                                                           GdkDragContext           *context,
-                                                           gint                      x,
-                                                           gint                      y,
-                                                           guint                     timestamp,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static gboolean thunar_renamer_dialog_drag_drop           (GtkWidget                *tree_view,
-                                                           GdkDragContext           *context,
-                                                           gint                      x,
-                                                           gint                      y,
-                                                           guint                     timestamp,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_notify_page         (GtkNotebook              *notebook,
-                                                           GParamSpec               *pspec,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static gboolean thunar_renamer_dialog_popup_menu          (GtkWidget                *tree_view,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_row_activated       (GtkTreeView              *tree_view,
-                                                           GtkTreePath              *path,
-                                                           GtkTreeViewColumn        *column,
-                                                           ThunarRenamerDialog      *renamer_dialog);
-static void     thunar_renamer_dialog_selection_changed   (GtkTreeSelection         *selection,
-                                                           ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_dispose               (GObject                  *object);
+static void        thunar_renamer_dialog_finalize              (GObject                  *object);
+static void        thunar_renamer_dialog_get_property          (GObject                  *object,
+                                                                guint                     prop_id,
+                                                                GValue                   *value,
+                                                                GParamSpec               *pspec);
+static void        thunar_renamer_dialog_set_property          (GObject                  *object,
+                                                                guint                     prop_id,
+                                                                const GValue             *value,
+                                                                GParamSpec               *pspec);
+static void        thunar_renamer_dialog_realize               (GtkWidget                *widget);
+static void        thunar_renamer_dialog_unrealize             (GtkWidget                *widget);
+static void        thunar_renamer_dialog_response              (GtkDialog                *dialog,
+                                                                gint                      response);
+static void        thunar_renamer_dialog_context_menu          (ThunarRenamerDialog      *renamer_dialog,
+                                                                guint                     button,
+                                                                guint32                   timestamp);
+static void        thunar_renamer_dialog_help                  (ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_save                  (ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_action_add_files      (GtkAction                *action,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_action_remove_files   (GtkAction                *action,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_action_clear          (GtkAction                *action,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_action_about          (GtkAction                *action,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_action_properties     (GtkAction                *action,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static gboolean    thunar_renamer_dialog_button_press_event    (GtkWidget                *tree_view,
+                                                                GdkEventButton           *event,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_drag_data_received    (GtkWidget                *tree_view,
+                                                                GdkDragContext           *context,
+                                                                gint                      x,
+                                                                gint                      y,
+                                                                GtkSelectionData         *selection_data,
+                                                                guint                     info,
+                                                                guint                     timestamp,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_drag_leave            (GtkWidget                *tree_view,
+                                                                GdkDragContext           *context,
+                                                                guint                     timestamp,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static gboolean    thunar_renamer_dialog_drag_motion           (GtkWidget                *tree_view,
+                                                                GdkDragContext           *context,
+                                                                gint                      x,
+                                                                gint                      y,
+                                                                guint                     timestamp,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static gboolean    thunar_renamer_dialog_drag_drop             (GtkWidget                *tree_view,
+                                                                GdkDragContext           *context,
+                                                                gint                      x,
+                                                                gint                      y,
+                                                                guint                     timestamp,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_notify_page           (GtkNotebook              *notebook,
+                                                                GParamSpec               *pspec,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static gboolean    thunar_renamer_dialog_popup_menu            (GtkWidget                *tree_view,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_row_activated         (GtkTreeView              *tree_view,
+                                                                GtkTreePath              *path,
+                                                                GtkTreeViewColumn        *column,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static void        thunar_renamer_dialog_selection_changed     (GtkTreeSelection         *selection,
+                                                                ThunarRenamerDialog      *renamer_dialog);
+static ThunarFile *thunar_renamer_dialog_get_current_directory (ThunarRenamerDialog *renamer_dialog);
+static void        thunar_renamer_dialog_set_current_directory (ThunarRenamerDialog *renamer_dialog,
+                                                                ThunarFile          *current_directory);
+static GList      *thunar_renamer_dialog_get_selected_files    (ThunarRenamerDialog *renamer_dialog);
+static gboolean    thunar_renamer_dialog_get_standalone        (ThunarRenamerDialog *renamer_dialog);
+static void        thunar_renamer_dialog_set_standalone        (ThunarRenamerDialog *renamer_dialog,
+                                                                gboolean             fixed);
 
 
 
@@ -336,7 +343,6 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   g_list_free (providers);
 
   /* initialize the dialog */
-  gtk_dialog_set_has_separator (GTK_DIALOG (renamer_dialog), FALSE);
   gtk_window_set_default_size (GTK_WINDOW (renamer_dialog), 510, 490);
   gtk_window_set_title (GTK_WINDOW (renamer_dialog), _("Rename Multiple Files"));
 
@@ -480,9 +486,9 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       xfce_rc_set_group (rc, "Configuration");
 
       /* create the renamer combo box for the renamer selection */
-      rcombo = gtk_combo_box_new_text ();
+      rcombo = gtk_combo_box_text_new ();
       for (lp = renamers; lp != NULL; lp = lp->next)
-        gtk_combo_box_append_text (GTK_COMBO_BOX (rcombo), thunarx_renamer_get_name (lp->data));
+        gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (rcombo), thunarx_renamer_get_name (lp->data));
       gtk_box_pack_start (GTK_BOX (rbox), rcombo, FALSE, FALSE, 0);
       gtk_widget_show (rcombo);
 
@@ -498,14 +504,14 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       gtk_widget_show (image);
 
       /* create the name/suffix/both combo box */
-      mcombo = gtk_combo_box_new_text ();
+      mcombo = gtk_combo_box_text_new ();
       klass = g_type_class_ref (THUNAR_TYPE_RENAMER_MODE);
       active_str = xfce_rc_read_entry_untranslated (rc, "LastActiveMode", "");
       for (active = 0, n = 0; n < klass->n_values; ++n)
         {
           if (exo_str_is_equal (active_str, klass->values[n].value_name))
             active = n;
-          gtk_combo_box_append_text (GTK_COMBO_BOX (mcombo), _(klass->values[n].value_nick));
+          gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (mcombo), _(klass->values[n].value_nick));
         }
       exo_mutual_binding_new (G_OBJECT (renamer_dialog->model), "mode", G_OBJECT (mcombo), "active");
       gtk_box_pack_end (GTK_BOX (rbox), mcombo, FALSE, FALSE, 0);
@@ -608,8 +614,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
     }
 
   /* release the renamers list */
-  g_list_foreach (renamers, (GFunc) g_object_unref, NULL);
-  g_list_free (renamers);
+  g_list_free_full (renamers, g_object_unref);
 }
 
 
@@ -1217,17 +1222,11 @@ thunar_renamer_dialog_action_properties (GtkAction           *action,
   _thunar_return_if_fail (GTK_IS_ACTION (action));
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
 
-  /* verify that we have exactly one file selected */
-  if (G_LIKELY (g_list_length (renamer_dialog->selected_files) == 1))
-    {
-      /* popup the properties dialog */
-      dialog = thunar_properties_dialog_new ();
-      gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-      gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
-      gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (renamer_dialog));
-      thunar_properties_dialog_set_file (THUNAR_PROPERTIES_DIALOG (dialog), renamer_dialog->selected_files->data);
-      gtk_widget_show (dialog);
-    }
+  /* popup the properties dialog */
+  dialog = thunar_properties_dialog_new (GTK_WINDOW (renamer_dialog));
+  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+  thunar_properties_dialog_set_files (THUNAR_PROPERTIES_DIALOG (dialog), renamer_dialog->selected_files);
+  gtk_widget_show (dialog);
 }
 
 
@@ -1524,8 +1523,7 @@ thunar_renamer_dialog_drag_drop (GtkWidget           *tree_view,
 
           /* perform the move */
           thunar_renamer_model_reorder (renamer_dialog->model, rows, position);
-          g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
-          g_list_free (rows);
+          g_list_free_full (rows, (GDestroyNotify) gtk_tree_path_free);
         }
 
       /* finish the dnd operation */
@@ -1651,7 +1649,7 @@ thunar_renamer_dialog_selection_changed (GtkTreeSelection    *selection,
 
   /* the "Properties" action is only sensitive if we have exactly one selected file */
   action = gtk_action_group_get_action (renamer_dialog->action_group, "properties");
-  gtk_action_set_sensitive (action, (n_selected_files == 1));
+  gtk_action_set_sensitive (action, n_selected_files > 0);
 
   /* notify listeners */
   g_object_notify (G_OBJECT (renamer_dialog), "selected-files");
@@ -1659,22 +1657,6 @@ thunar_renamer_dialog_selection_changed (GtkTreeSelection    *selection,
 
 
 
-/**
- * thunar_renamer_dialog_new:
- *
- * Allocates a new #ThunarRenamerDialog instance.
- *
- * Return value: the newly allocated #ThunarRenamerDialog
- *               instance.
- **/
-GtkWidget*
-thunar_renamer_dialog_new (void)
-{
-  return g_object_new (THUNAR_TYPE_RENAMER_DIALOG, NULL);
-}
-
-
-
 /**
  * thunar_renamer_dialog_get_current_directory:
  * @renamer_dialog : a #ThunarRenamerDialog.
@@ -1686,7 +1668,7 @@ thunar_renamer_dialog_new (void)
  *
  * Return value: the current directory for @renamer_dialog.
  **/
-ThunarFile*
+static ThunarFile*
 thunar_renamer_dialog_get_current_directory (ThunarRenamerDialog *renamer_dialog)
 {
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), NULL);
@@ -1705,7 +1687,7 @@ thunar_renamer_dialog_get_current_directory (ThunarRenamerDialog *renamer_dialog
  * is %NULL, the current directory of the file manager process
  * will be used.
  **/
-void
+static void
 thunar_renamer_dialog_set_current_directory (ThunarRenamerDialog *renamer_dialog,
                                              ThunarFile          *current_directory)
 {
@@ -1742,7 +1724,7 @@ thunar_renamer_dialog_set_current_directory (ThunarRenamerDialog *renamer_dialog
  *
  * Return value: the list of currently selected files.
  **/
-GList*
+static GList*
 thunar_renamer_dialog_get_selected_files (ThunarRenamerDialog *renamer_dialog)
 {
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), NULL);
@@ -1761,7 +1743,7 @@ thunar_renamer_dialog_get_selected_files (ThunarRenamerDialog *renamer_dialog)
  *
  * Return value: %TRUE if @renamer_dialog is standalone.
  **/
-gboolean
+static gboolean
 thunar_renamer_dialog_get_standalone (ThunarRenamerDialog *renamer_dialog)
 {
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), FALSE);
@@ -1779,7 +1761,7 @@ thunar_renamer_dialog_get_standalone (ThunarRenamerDialog *renamer_dialog)
  * toolbar and several other actions that do not make sense
  * when opened from within the file manager.
  **/
-void
+static void
 thunar_renamer_dialog_set_standalone (ThunarRenamerDialog *renamer_dialog,
                                       gboolean             standalone)
 {
@@ -1880,7 +1862,7 @@ thunar_show_renamer_dialog (gpointer     parent,
     gtk_window_set_startup_id (GTK_WINDOW (dialog), startup_id);
 
   /* check if we have a toplevel window */
-  if (G_LIKELY (window != NULL && GTK_WIDGET_TOPLEVEL (window)))
+  if (G_LIKELY (window != NULL && gtk_widget_get_toplevel (window)))
     {
       /* dialog is transient for toplevel window, but not modal */
       gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
