@@ -1,6 +1,7 @@
-/* $Id: thunar-renamer-model.c 24208 2006-12-28 21:25:19Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -49,15 +50,20 @@ enum
   PROP_RENAMER,
 };
 
+typedef struct
+{
+  gint   offset;
+  gint   position;
+  GList *item;
+} SortTuple;
+
 
 
 typedef struct _ThunarRenamerModelItem ThunarRenamerModelItem;
 
 
 
-static void                    thunar_renamer_model_class_init          (ThunarRenamerModelClass *klass);
 static void                    thunar_renamer_model_tree_model_init     (GtkTreeModelIface       *iface);
-static void                    thunar_renamer_model_init                (ThunarRenamerModel      *renamer_model);
 static void                    thunar_renamer_model_finalize            (GObject                 *object);
 static void                    thunar_renamer_model_get_property        (GObject                 *object,
                                                                          guint                    prop_id,
@@ -109,11 +115,20 @@ static gboolean                thunar_renamer_model_conflict_item       (ThunarR
                                                                          ThunarRenamerModelItem  *item);
 static gchar                  *thunar_renamer_model_process_item        (ThunarRenamerModel      *renamer_model,
                                                                          ThunarRenamerModelItem  *item,
-                                                                         guint                    index);
+                                                                         guint                    idx);
 static gboolean                thunar_renamer_model_update_idle         (gpointer                 user_data);
 static void                    thunar_renamer_model_update_idle_destroy (gpointer                 user_data);
 static ThunarRenamerModelItem *thunar_renamer_model_item_new            (ThunarFile              *file) G_GNUC_MALLOC;
-static void                    thunar_renamer_model_item_free           (ThunarRenamerModelItem  *item);
+static void                    thunar_renamer_model_item_free           (gpointer                 data);
+static gint                    thunar_renamer_model_cmp_array           (gconstpointer            pointer_a,
+                                                                         gconstpointer            pointer_b,
+                                                                         gpointer                 user_data);
+static gboolean                thunar_renamer_model_get_can_rename      (ThunarRenamerModel      *renamer_model);
+static gboolean                thunar_renamer_model_get_frozen          (ThunarRenamerModel      *renamer_model);
+static void                    thunar_renamer_model_set_frozen          (ThunarRenamerModel      *renamer_model,
+                                                                         gboolean                 frozen);
+static void                    thunar_renamer_model_set_mode            (ThunarRenamerModel      *renamer_model,
+                                                                         ThunarRenamerMode        mode);
 
 
 
@@ -131,7 +146,7 @@ struct _ThunarRenamerModel
    * generated by another model.
    */
 #ifndef NDEBUG
-  guint              stamp;
+  gint               stamp;
 #endif
 
   ThunarRenamerMode  mode;
@@ -148,54 +163,18 @@ struct _ThunarRenamerModel
 
 struct _ThunarRenamerModelItem
 {
-  ThunarVfsInfo *info;
-  ThunarFile    *file;
-  gchar         *name;
-  guint          changed : 1;  /* if the file changed */
-  guint          conflict : 1; /* if the item conflicts with another item */
-  guint          dirty : 1;    /* if the item must be updated */
+  ThunarFile *file;
+  gchar      *name;
+  guint64     date_changed;
+  guint       changed : 1;  /* if the file changed */
+  guint       conflict : 1; /* if the item conflicts with another item */
+  guint       dirty : 1;    /* if the item must be updated */
 };
 
 
 
-static GObjectClass *thunar_renamer_model_parent_class;
-
-
-
-GType
-thunar_renamer_model_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarRenamerModelClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_renamer_model_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarRenamerModel),
-        0,
-        (GInstanceInitFunc) thunar_renamer_model_init,
-        NULL,
-      };
-
-      static const GInterfaceInfo tree_model_info =
-      {
-        (GInterfaceInitFunc) thunar_renamer_model_tree_model_init,
-        NULL,
-        NULL,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, I_("ThunarRenamerModel"), &info, 0);
-      g_type_add_interface_static (type, GTK_TYPE_TREE_MODEL, &tree_model_info);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE_WITH_CODE (ThunarRenamerModel, thunar_renamer_model, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL, thunar_renamer_model_tree_model_init))
 
 
 
@@ -204,9 +183,6 @@ thunar_renamer_model_class_init (ThunarRenamerModelClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_renamer_model_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_renamer_model_finalize;
   gobject_class->get_property = thunar_renamer_model_get_property;
@@ -318,8 +294,7 @@ thunar_renamer_model_finalize (GObject *object)
   thunar_renamer_model_set_renamer (renamer_model, NULL);
 
   /* release all items */
-  g_list_foreach (renamer_model->items, (GFunc) thunar_renamer_model_item_free, NULL);
-  g_list_free (renamer_model->items);
+  g_list_free_full (renamer_model->items, thunar_renamer_model_item_free);
 
   /* disconnect from the file monitor */
   g_signal_handlers_disconnect_by_func (G_OBJECT (renamer_model->file_monitor), thunar_renamer_model_file_destroyed, renamer_model);
@@ -469,17 +444,17 @@ thunar_renamer_model_get_path (GtkTreeModel *tree_model,
                                GtkTreeIter  *iter)
 {
   ThunarRenamerModel *renamer_model = THUNAR_RENAMER_MODEL (tree_model);
-  gint                index;
+  gint                idx;
 
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_MODEL (renamer_model), NULL);
   _thunar_return_val_if_fail (iter->stamp == renamer_model->stamp, NULL);
 
-  /* determine the index of the item */
-  index = g_list_position (renamer_model->items, iter->user_data);
-  if (G_UNLIKELY (index < 0))
+  /* determine the idx of the item */
+  idx = g_list_position (renamer_model->items, iter->user_data);
+  if (G_UNLIKELY (idx < 0))
     return NULL;
 
-  return gtk_tree_path_new_from_indices (index, -1);
+  return gtk_tree_path_new_from_indices (idx, -1);
 }
 
 
@@ -623,6 +598,7 @@ thunar_renamer_model_file_changed (ThunarRenamerModel *renamer_model,
   GtkTreePath            *path;
   GtkTreeIter             iter;
   GList                  *lp;
+  guint64                 date_changed;
 
   _thunar_return_if_fail (THUNAR_IS_FILE (file));
   _thunar_return_if_fail (THUNAR_IS_FILE_MONITOR (file_monitor));
@@ -633,24 +609,27 @@ thunar_renamer_model_file_changed (ThunarRenamerModel *renamer_model,
   for (lp = renamer_model->items; lp != NULL; lp = lp->next)
     if (THUNAR_RENAMER_MODEL_ITEM (lp->data)->file == file)
       {
-        /* check if the file changed on disk */
         item = THUNAR_RENAMER_MODEL_ITEM (lp->data);
-        if (!thunar_vfs_info_matches (item->info, thunar_file_get_info (file)))
+
+        /* check if the file changed on disk, this is done to prevent
+         * excessive looping when some renamers are used
+         * (thunar-media-tags-plugin is an example) */
+        date_changed = thunar_file_get_date (file, THUNAR_FILE_DATE_CHANGED);
+        if (item->date_changed == date_changed)
+          break;
+
+        /* check if we're frozen */
+        if (G_LIKELY (!renamer_model->frozen))
           {
-            /* connect to the new info */
-            thunar_vfs_info_unref (item->info);
-            item->info = thunar_vfs_info_ref (thunar_file_get_info (file));
+            /* the file changed */
+            item->changed = TRUE;
 
-            /* check if we're frozen */
-            if (G_LIKELY (!renamer_model->frozen))
-              {
-                /* the file changed */
-                THUNAR_RENAMER_MODEL_ITEM (lp->data)->changed = TRUE;
+            /* set the new mtime */
+            item->date_changed = date_changed;
 
-                /* invalidate the item */
-                thunar_renamer_model_invalidate_item (renamer_model, lp->data);
-                break;
-              }
+            /* invalidate the item */
+            thunar_renamer_model_invalidate_item (renamer_model, item);
+            break;
           }
 
         /* determine the iter for the item */
@@ -673,7 +652,7 @@ thunar_renamer_model_file_destroyed (ThunarRenamerModel *renamer_model,
 {
   GtkTreePath *path;
   GList       *lp;
-  gint         index;
+  gint         idx;
 
   _thunar_return_if_fail (THUNAR_IS_RENAMER_MODEL (renamer_model));
   _thunar_return_if_fail (renamer_model->file_monitor == file_monitor);
@@ -684,8 +663,8 @@ thunar_renamer_model_file_destroyed (ThunarRenamerModel *renamer_model,
   for (lp = renamer_model->items; lp != NULL; lp = lp->next)
     if (THUNAR_RENAMER_MODEL_ITEM (lp->data)->file == file)
       {
-        /* determine the index of the item */
-        index = g_list_position (renamer_model->items, lp);
+        /* determine the idx of the item */
+        idx = g_list_position (renamer_model->items, lp);
 
         /* free the item data */
         thunar_renamer_model_item_free (lp->data);
@@ -694,7 +673,7 @@ thunar_renamer_model_file_destroyed (ThunarRenamerModel *renamer_model,
         renamer_model->items = g_list_delete_link (renamer_model->items, lp);
 
         /* tell the view that the item is gone */
-        path = gtk_tree_path_new_from_indices (index, -1);
+        path = gtk_tree_path_new_from_indices (idx, -1);
         gtk_tree_model_row_deleted (GTK_TREE_MODEL (renamer_model), path);
         gtk_tree_path_free (path);
 
@@ -743,15 +722,15 @@ static gboolean
 trm_same_directory (ThunarFile *a,
                     ThunarFile *b)
 {
-  ThunarVfsPath *parent_a;
-  ThunarVfsPath *parent_b;
+  GFile *parent_a;
+  GFile *parent_b;
 
   /* determine the parent paths for both files */
-  parent_a = thunar_vfs_path_get_parent (thunar_file_get_path (a));
-  parent_b = thunar_vfs_path_get_parent (thunar_file_get_path (b));
+  parent_a = g_file_get_parent (thunar_file_get_file (a));
+  parent_b = g_file_get_parent (thunar_file_get_file (b));
 
   /* check if both files have the same parent */
-  return (parent_a != NULL && parent_b != NULL && thunar_vfs_path_equal (parent_a, parent_b));
+  return (parent_a != NULL && parent_b != NULL && g_file_equal (parent_a, parent_b));
 }
 
 
@@ -807,7 +786,7 @@ thunar_renamer_model_conflict_item (ThunarRenamerModel     *renamer_model,
 static gchar*
 thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
                                    ThunarRenamerModelItem *item,
-                                   guint                   index)
+                                   guint                   idx)
 {
   ThunarRenamerMode mode;
   const gchar      *display_name;
@@ -846,7 +825,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
           text = g_strndup (display_name, (dot - display_name));
 
           /* determine the new name */
-          prefix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), text, index);
+          prefix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), text, idx);
 
           /* determine the new full name */
           name = g_strconcat (prefix, dot, NULL);
@@ -858,12 +837,11 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
 
         case THUNAR_RENAMER_MODE_SUFFIX:
           /* determine the new suffix */
-          suffix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), dot + 1, index);
+          suffix = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), dot + 1, idx);
 
-          /* generate the new file name */
-          name = g_new (gchar, (dot - display_name) + 1 + strlen (suffix) + 1);
-          memcpy (name, display_name, (dot - display_name) + 1);
-          memcpy (name + (dot - display_name) + 1, suffix, strlen (suffix) + 1);
+          prefix = g_strndup (display_name, (dot - display_name) + 1);
+          name = g_strconcat (prefix, suffix, NULL);
+          g_free (prefix);
 
           /* release the suffix */
           g_free (suffix);
@@ -871,7 +849,7 @@ thunar_renamer_model_process_item (ThunarRenamerModel     *renamer_model,
 
         case THUNAR_RENAMER_MODE_BOTH:
           /* determine the new full name */
-          name = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), display_name, index);
+          name = thunarx_renamer_process (renamer_model->renamer, THUNARX_FILE_INFO (item->file), display_name, idx);
           break;
 
         default:
@@ -902,7 +880,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
   GtkTreeIter             iter;
   gboolean                changed = FALSE;
   gboolean                conflict;
-  guint                   index;
+  guint                   idx;
   gchar                  *name;
   GList                  *lp;
 
@@ -912,7 +890,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
   if (G_LIKELY (!renamer_model->frozen))
     {
       /* process the first dirty item */
-      for (index = 0, lp = renamer_model->items; !changed && lp != NULL; ++index, lp = lp->next)
+      for (idx = 0, lp = renamer_model->items; !changed && lp != NULL; ++idx, lp = lp->next)
         {
           /* check if this item is dirty */
           item = THUNAR_RENAMER_MODEL_ITEM (lp->data);
@@ -927,7 +905,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
           item->dirty = FALSE;
 
           /* determine the new name for the item */
-          name = thunar_renamer_model_process_item (renamer_model, item, index);
+          name = thunar_renamer_model_process_item (renamer_model, item, idx);
           if (!exo_str_is_equal (item->name, name))
             {
               /* apply new name */
@@ -961,7 +939,7 @@ thunar_renamer_model_update_idle (gpointer user_data)
               GTK_TREE_ITER_INIT (iter, renamer_model->stamp, lp);
 
               /* emit "row-changed" for this item */
-              path = gtk_tree_path_new_from_indices (index, -1);
+              path = gtk_tree_path_new_from_indices (idx, -1);
               gtk_tree_model_row_changed (GTK_TREE_MODEL (renamer_model), path, &iter);
               gtk_tree_path_free (path);
             }
@@ -993,9 +971,9 @@ thunar_renamer_model_item_new (ThunarFile *file)
 {
   ThunarRenamerModelItem *item;
 
-  item = _thunar_slice_new0 (ThunarRenamerModelItem);
-  item->info = thunar_vfs_info_ref (thunar_file_get_info (file));
+  item = g_slice_new0 (ThunarRenamerModelItem);
   item->file = g_object_ref (G_OBJECT (file));
+  item->date_changed = thunar_file_get_date (file, THUNAR_FILE_DATE_CHANGED);
   item->dirty = TRUE;
 
   return item;
@@ -1004,12 +982,31 @@ thunar_renamer_model_item_new (ThunarFile *file)
 
 
 static void
-thunar_renamer_model_item_free (ThunarRenamerModelItem *item)
+thunar_renamer_model_item_free (gpointer data)
 {
+  ThunarRenamerModelItem *item = data;
+
   g_object_unref (G_OBJECT (item->file));
-  thunar_vfs_info_unref (item->info);
   g_free (item->name);
-  _thunar_slice_free (ThunarRenamerModelItem, item);
+  g_slice_free (ThunarRenamerModelItem, item);
+}
+
+
+
+static gint
+thunar_renamer_model_cmp_array (gconstpointer pointer_a,
+                                gconstpointer pointer_b,
+                                gpointer      user_data)
+{
+  const SortTuple *a = pointer_a;
+  const SortTuple *b = pointer_b;
+  
+  if (G_UNLIKELY (a->position == b->position))
+    /* return the sort order for the 'moved' items in the list */
+    return a->offset - b->offset;
+  else
+    /* return the sort order for all other items in the lis */
+    return a->position - b->position;
 }
 
 
@@ -1043,7 +1040,7 @@ thunar_renamer_model_new (void)
  *
  * Return value: %TRUE if bulk rename can be performed.
  **/
-gboolean
+static gboolean
 thunar_renamer_model_get_can_rename (ThunarRenamerModel *renamer_model)
 {
   gboolean can_rename = FALSE;
@@ -1081,7 +1078,7 @@ thunar_renamer_model_get_can_rename (ThunarRenamerModel *renamer_model)
  *
  * Return value: %TRUE if @renamer_model is frozen.
  **/
-gboolean
+static gboolean
 thunar_renamer_model_get_frozen (ThunarRenamerModel *renamer_model)
 {
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_MODEL (renamer_model), FALSE);
@@ -1099,7 +1096,7 @@ thunar_renamer_model_get_frozen (ThunarRenamerModel *renamer_model)
  * and no updates will be processed until you call this method again with a
  * value of %FALSE for @frozen.
  **/
-void
+static void
 thunar_renamer_model_set_frozen (ThunarRenamerModel *renamer_model,
                                  gboolean            frozen)
 {
@@ -1163,7 +1160,7 @@ thunar_renamer_model_get_mode (ThunarRenamerModel *renamer_model)
  * Sets the rename operation mode of @renamer_model
  * to @mode and updates all items.
  **/
-void
+static void
 thunar_renamer_model_set_mode (ThunarRenamerModel *renamer_model,
                                ThunarRenamerMode   mode)
 {
@@ -1249,15 +1246,17 @@ thunar_renamer_model_set_renamer (ThunarRenamerModel *renamer_model,
 
 
 /**
- * thunar_renamer_model_append:
+ * thunar_renamer_model_insert:
  * @renamer_model : a #ThunarRenamerModel.
- * @file              : the #ThunarFile to add to @renamer_model.
+ * @file          : the #ThunarFile to add to @renamer_model.
+ * @position      : the position in the model. 0 is prepend, -1 is append.
  *
- * Adds the @file to the @renamer_model.
+ * Inserts the @file to the @renamer_model at @position.
  **/
 void
-thunar_renamer_model_append (ThunarRenamerModel *renamer_model,
-                             ThunarFile         *file)
+thunar_renamer_model_insert (ThunarRenamerModel *renamer_model,
+                             ThunarFile         *file,
+                             gint                position)
 {
   ThunarRenamerModelItem *item;
   GtkTreePath            *path;
@@ -1276,10 +1275,10 @@ thunar_renamer_model_append (ThunarRenamerModel *renamer_model,
   item = thunar_renamer_model_item_new (file);
 
   /* append the item to the model */
-  renamer_model->items = g_list_append (renamer_model->items, item);
+  renamer_model->items = g_list_insert (renamer_model->items, item, position);
 
   /* determine the iterator for the new item */
-  GTK_TREE_ITER_INIT (iter, renamer_model->stamp, g_list_last (renamer_model->items));
+  GTK_TREE_ITER_INIT (iter, renamer_model->stamp, g_list_find (renamer_model->items, item));
 
   /* emit the "row-inserted" signal */
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (renamer_model), &iter);
@@ -1292,6 +1291,102 @@ thunar_renamer_model_append (ThunarRenamerModel *renamer_model,
 
 
 
+/**
+ * thunar_renamer_model_reorder:
+ * @renamer_model : a #ThunarRenamerModel.
+ * @tree_paths    : the list of #GtkTreePath that need to be moved to
+ *                  @position.
+ * @position      : the new position for the list of paths.
+ * 
+ * Reorder the treepaths in the model to their new postion in the list
+ * and sends an update to the treeview to reorder the list.
+ **/
+void
+thunar_renamer_model_reorder (ThunarRenamerModel *renamer_model,
+                              GList              *tree_paths,
+                              gint                position)
+{
+  GList       *lp, *lprev;
+  gint         n_items;
+  gint        *new_order;
+  GtkTreePath *path;
+  gint         k, n, m;
+  SortTuple   *sort_array;
+
+  /* leave when there is nothing to sort */
+  n_items = g_list_length (renamer_model->items);
+  if (G_UNLIKELY (n_items <= 1))
+    return;
+
+  /* correct the sort position to match the list items */
+  if (position == -1 || position > n_items)
+    position = n_items;
+
+  /* be sure to not overuse the stack */
+  if (G_LIKELY (n_items < 500))
+    sort_array = g_newa (SortTuple, n_items);
+  else
+    sort_array = g_new (SortTuple, n_items);
+
+  /* generate the sort array of tuples */
+  for (lp = renamer_model->items, m = 0, n = 0; lp != NULL; lp = lp->next, ++n, ++m)
+    {
+      /* leave a hole in the sort position for the drop items */
+      if (G_UNLIKELY (m == position))
+        m++;
+
+      sort_array[n].offset = n;
+      sort_array[n].item = lp;
+      sort_array[n].position = m;
+    }
+
+  /* set the requested sort position for the dragged paths */
+  for (lp = tree_paths; lp != NULL; lp = lp->next)
+    {
+      k = CLAMP (gtk_tree_path_get_indices (lp->data)[0], 0, n_items - 1);
+      sort_array[k].position = position;
+    }
+
+  /* sort the array using QuickSort */
+  g_qsort_with_data (sort_array, n_items, sizeof (SortTuple), thunar_renamer_model_cmp_array, NULL);
+
+  /* update our internals and generate the new order */
+  new_order = g_newa (gint, n_items);
+  for (n = 0, lprev = NULL; n < n_items; ++n)
+    {
+      /* set the new order in the sort list */
+      new_order[n] = sort_array[n].offset;
+
+      /* reinsert the item in the list */
+      lp = sort_array[n].item;
+      lp->next = NULL;
+      lp->prev = lprev;
+
+      /* hookup the item in the list */
+      if (G_LIKELY (lprev != NULL))
+        lprev->next = lp;
+      else
+        renamer_model->items = lp;
+
+      /* advance the offset */
+      lprev = lp;
+    }
+
+  /* tell the view about the new item order */
+  path = gtk_tree_path_new ();
+  gtk_tree_model_rows_reordered (GTK_TREE_MODEL (renamer_model), path, NULL, new_order);
+  gtk_tree_path_free (path);
+
+  /* invalidate all items */
+  thunar_renamer_model_invalidate_all (renamer_model);
+  
+  /* cleanup if we used the heap */
+  if (G_UNLIKELY (n_items >= 500))
+    g_free (sort_array);
+}
+
+
+
 /**
  * thunar_renamer_model_clear:
  * @renamer_model : a #ThunarRenamerModel.
