@@ -271,7 +271,7 @@ thunar_shortcuts_model_init (ThunarShortcutsModel *model)
                    G_OBJECT (model), "hidden-bookmarks");
 
   /* load volumes */
-  model->devices_monitor_idle_id = g_idle_add (thunar_shortcuts_model_devices_load, model);
+  model->devices_monitor_idle_id = g_idle_add_full (G_PRIORITY_DEFAULT, thunar_shortcuts_model_devices_load, model, NULL);
 
   /* add network */
   thunar_shortcuts_model_shortcut_network (model);
@@ -564,7 +564,7 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
       else if (shortcut->file != NULL)
         g_value_set_static_string (value, thunar_file_get_display_name (shortcut->file));
       else if (shortcut->location != NULL)
-        g_value_take_string (value, thunar_g_file_get_display_name (shortcut->location));
+        g_value_take_string (value, thunar_g_file_get_display_name_remote (shortcut->location));
       else
         g_value_set_static_string (value, "");
       break;
@@ -935,7 +935,7 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
     }
 
   /* read the Gtk+ bookmarks file */
-  model->bookmarks_idle_id = g_idle_add (thunar_shortcuts_model_load, model);
+  model->bookmarks_idle_id = g_idle_add_full (G_PRIORITY_DEFAULT, thunar_shortcuts_model_load, model, NULL);
 
   g_object_unref (home);
 }
@@ -994,8 +994,7 @@ thunar_shortcuts_model_sort_func (gconstpointer shortcut_a,
 
   /* properly sort devices by timestamp */
   if (a->device != NULL && b->device != NULL)
-    return -g_strcmp0 (thunar_device_get_sort_key (a->device),
-                       thunar_device_get_sort_key (b->device));
+    return thunar_device_sort (a->device, b->device);
 
   return g_strcmp0 (a->name, b->name);
 }
@@ -1617,6 +1616,49 @@ thunar_shortcuts_model_get_default (void)
 
 
 
+/**
+ * thunar_shortcuts_model_has_bookmark:
+ * @model : a #ThunarShortcutsModel instance.
+ * @file  : a #ThuanrFile instance.
+ *
+ * Returns %TRUE if there is a bookmark (not a mount or volume) with
+ * @file as destination.
+ *
+ * Return value: %TRUE if @file was found, else %FALSE.
+ **/
+gboolean
+thunar_shortcuts_model_has_bookmark (ThunarShortcutsModel *model,
+                                     GFile                *file)
+{
+  GList          *lp;
+  ThunarShortcut *shortcut;
+
+  _thunar_return_val_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model), FALSE);
+  _thunar_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+  for (lp = model->shortcuts; lp != NULL; lp = lp->next)
+    {
+      shortcut = lp->data;
+
+      /* only check bookmarks */
+      if (shortcut->group != THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
+          && shortcut->group != THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+        continue;
+
+      if (shortcut->file != NULL
+          && g_file_equal (thunar_file_get_file (shortcut->file), file))
+        return TRUE;
+
+      if (shortcut->location != NULL
+          && g_file_equal (shortcut->location, file))
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
+
+
 /**
  * thunar_shortcuts_model_iter_for_file:
  * @model : a #ThunarShortcutsModel instance.
@@ -1737,27 +1779,32 @@ thunar_shortcuts_model_drop_possible (ThunarShortcutsModel *model,
 void
 thunar_shortcuts_model_add (ThunarShortcutsModel *model,
                             GtkTreePath          *dst_path,
-                            ThunarFile           *file)
+                            gpointer              file)
 {
   ThunarShortcut *shortcut;
-  GList           *lp;
+  GFile          *location;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
-  _thunar_return_if_fail (gtk_tree_path_get_depth (dst_path) > 0);
-  _thunar_return_if_fail (gtk_tree_path_get_indices (dst_path)[0] >= 0);
-  _thunar_return_if_fail (gtk_tree_path_get_indices (dst_path)[0] <= (gint) g_list_length (model->shortcuts));
-  _thunar_return_if_fail (THUNAR_IS_FILE (file));
+  _thunar_return_if_fail (dst_path == NULL || gtk_tree_path_get_depth (dst_path) > 0);
+  _thunar_return_if_fail (dst_path == NULL || gtk_tree_path_get_indices (dst_path)[0] >= 0);
+  _thunar_return_if_fail (dst_path == NULL || gtk_tree_path_get_indices (dst_path)[0] <= (gint) g_list_length (model->shortcuts));
+  _thunar_return_if_fail (THUNAR_IS_FILE (file) || G_IS_FILE (file));
+
+  location = G_IS_FILE (file) ? file : thunar_file_get_file (file);
 
   /* verify that the file is not already in use as shortcut */
-  for (lp = model->shortcuts; lp != NULL; lp = lp->next)
-    if (THUNAR_SHORTCUT (lp->data)->file == file)
-      return;
+  if (thunar_shortcuts_model_has_bookmark (model, location))
+    return;
 
   /* create the new shortcut that will be inserted */
   shortcut = g_slice_new0 (ThunarShortcut);
-  shortcut->file = g_object_ref (G_OBJECT (file));
 
-  if (thunar_file_is_local (file))
+  if (THUNAR_IS_FILE (file) && thunar_file_is_local (file))
+    shortcut->file = g_object_ref (G_OBJECT (file));
+  else
+    shortcut->location = g_object_ref (G_OBJECT (location));
+
+  if (g_file_has_uri_scheme (location, "file"))
     {
       shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
     }
