@@ -45,6 +45,7 @@
 #include <thunar/thunar-shortcuts-model.h>
 #include <thunar/thunar-shortcuts-view.h>
 #include <thunar/thunar-device-monitor.h>
+#include <thunar/thunar-stock.h>
 
 
 
@@ -52,6 +53,7 @@
 enum
 {
   SHORTCUT_ACTIVATED,
+  SHORTCUT_ACTIVATED_TAB,
   LAST_SIGNAL,
 };
 
@@ -62,6 +64,15 @@ enum
   TEXT_URI_LIST,
 };
 
+/* Target for open action */
+typedef enum
+{
+  OPEN_IN_VIEW,
+  OPEN_IN_WINDOW,
+  OPEN_IN_TAB
+}
+OpenTarget;
+
 
 
 static void           thunar_shortcuts_view_finalize                     (GObject                  *object);
@@ -129,9 +140,11 @@ static void           thunar_shortcuts_view_drop_uri_list                (Thunar
                                                                           GtkTreePath              *dst_path);
 static void           thunar_shortcuts_view_open_clicked                 (ThunarShortcutsView      *view);
 static void           thunar_shortcuts_view_open                         (ThunarShortcutsView      *view,
-                                                                          gboolean                  new_window);
+                                                                          OpenTarget                open_in);
 static void           thunar_shortcuts_view_open_in_new_window_clicked   (ThunarShortcutsView      *view);
+static void           thunar_shortcuts_view_open_in_new_tab_clicked      (ThunarShortcutsView      *view);
 static void           thunar_shortcuts_view_empty_trash                  (ThunarShortcutsView      *view);
+static void           thunar_shortcuts_view_create_shortcut              (ThunarShortcutsView      *view);
 static void           thunar_shortcuts_view_eject                        (ThunarShortcutsView      *view);
 static void           thunar_shortcuts_view_mount                        (ThunarShortcutsView      *view);
 static void           thunar_shortcuts_view_unmount                      (ThunarShortcutsView      *view);
@@ -229,6 +242,19 @@ thunar_shortcuts_view_class_init (ThunarShortcutsViewClass *klass)
                   0, NULL, NULL,
                   g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, THUNAR_TYPE_FILE);
+
+  /**
+   * ThunarShortcutsView:shortcut-activated-tab:
+   *
+   * Invoked whenever a shortcut is activated by the user and should be opened in a new tab,
+   **/
+  view_signals[SHORTCUT_ACTIVATED_TAB] =
+    g_signal_new (I_("shortcut-activated-tab"),
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_VOID__OBJECT,
+                  G_TYPE_NONE, 1, THUNAR_TYPE_FILE);
 }
 
 
@@ -382,6 +408,7 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
   GtkTreeIter          iter;
   gboolean             result;
   gboolean             can_eject;
+  gint                 icon_width, icon_height, column_width;
 
   /* reset the pressed button state */
   view->pressed_button = -1;
@@ -411,11 +438,11 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
               result = TRUE;
             }
         }
-      else if ((event->button == 1 || event->button == 2) && event->type == GDK_BUTTON_PRESS
-            && (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
+      else if ((event->button == 1 || event->button == 2)
+               && event->type == GDK_BUTTON_PRESS
+               && (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
         {
           /* check if we clicked the eject button area */
-          gint icon_width, icon_height, column_width;
           column_width = gtk_tree_view_column_get_width (gtk_tree_view_get_column (GTK_TREE_VIEW (view), 0));
           gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &icon_width, &icon_height);
           if (event->x >= column_width - icon_width - 3)
@@ -430,15 +457,8 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
                 }
             }
 
-          /*
-          g_debug("thunar_shortcuts_view_button_press_event(): x: %f, y: %f; my width: %i, eject width: %i, eject: %i",
-                  event->x, event->y, column_width, icon_width, view->pressed_eject_button);
-          */
-
           /* remember the button as pressed and handle it in the release handler */
           view->pressed_button = event->button;
-
-
         }
 
       /* release the path */
@@ -455,18 +475,27 @@ thunar_shortcuts_view_button_release_event (GtkWidget      *widget,
                                             GdkEventButton *event)
 {
   ThunarShortcutsView *view = THUNAR_SHORTCUTS_VIEW (widget);
+  gboolean             in_tab;
 
   /* check if we have an event matching the pressed button state */
   if (G_LIKELY (view->pressed_button == (gint) event->button))
     {
+      /* check if the eject button has been pressed */
       if (view->pressed_eject_button)
-        thunar_shortcuts_view_eject (view);
-
-      /* check if we should simply open or open in new window */
-      if (G_LIKELY (event->button == 1))
-        thunar_shortcuts_view_open (view, FALSE);
+        {
+          thunar_shortcuts_view_eject (view);
+        }
+      else if (G_LIKELY (event->button == 1))
+        {
+          /* button 1 opens in the same window */
+          thunar_shortcuts_view_open (view, OPEN_IN_VIEW);
+        }
       else if (G_UNLIKELY (event->button == 2))
-        thunar_shortcuts_view_open (view, TRUE);
+        {
+          /* button 2 opens in a new window or tab */
+          g_object_get (view->preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+          thunar_shortcuts_view_open (view, in_tab ? OPEN_IN_TAB : OPEN_IN_WINDOW);
+        }
     }
 
   /* reset the pressed button state */
@@ -492,7 +521,7 @@ thunar_shortcuts_view_key_release_event (GtkWidget   *widget,
     case GDK_Down:
     case GDK_KP_Up:
     case GDK_KP_Down:
-      thunar_shortcuts_view_open (view, FALSE);
+      thunar_shortcuts_view_open (view, OPEN_IN_VIEW);
 
       /* keep focus on us */
       gtk_widget_grab_focus (widget);
@@ -866,7 +895,7 @@ thunar_shortcuts_view_row_activated (GtkTreeView       *tree_view,
     (*GTK_TREE_VIEW_CLASS (thunar_shortcuts_view_parent_class)->row_activated) (tree_view, path, column);
 
   /* open the selected shortcut */
-  thunar_shortcuts_view_open (view, FALSE);
+  thunar_shortcuts_view_open (view, OPEN_IN_VIEW);
 }
 
 
@@ -1017,6 +1046,8 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
   GList               *actions = NULL, *tmp;
   ThunarShortcutGroup  group;
   gboolean             is_header;
+  GFile               *mount_point;
+  GtkTreeModel        *shortcuts_model;
 
   /* check if this is an item menu or a header menu */
   gtk_tree_model_get (model, iter, THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER, &is_header, -1);
@@ -1051,6 +1082,12 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
   /* set the stock icon */
   image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+  
+  /* append the "Open in New Tab" menu action */
+  item = gtk_image_menu_item_new_with_mnemonic (_("Open in New Tab"));
+  g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_open_in_new_tab_clicked), view);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+  gtk_widget_show (item);
 
   /* append the "Open in New Window" menu action */
   item = gtk_image_menu_item_new_with_mnemonic (_("Open in New Window"));
@@ -1085,19 +1122,46 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
         g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_eject), view);
         break;
 
-      case THUNAR_SHORTCUT_GROUP_DEVICES_MOUNTS:
       case THUNAR_SHORTCUT_GROUP_NETWORK_MOUNTS:
         /* append a menu separator */
         item = gtk_separator_menu_item_new ();
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         gtk_widget_show (item);
 
+        /* get the mount point */
+        mount_point = thunar_device_get_root (device);
+        shortcuts_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
+
+        /* append the "Disconnect" item */
+        item = gtk_image_menu_item_new_with_mnemonic (_("Create _Shortcut"));
+        gtk_widget_set_sensitive (item, mount_point != NULL && !thunar_shortcuts_model_has_bookmark (THUNAR_SHORTCUTS_MODEL (shortcuts_model), mount_point));
+        g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_create_shortcut), view);
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+        gtk_widget_show (item);
+
+        image = gtk_image_new_from_stock (THUNAR_STOCK_SHORTCUTS, GTK_ICON_SIZE_MENU);
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+        if (mount_point != NULL)
+          g_object_unref (mount_point);
+
+        /* fall-through */
+
+      case THUNAR_SHORTCUT_GROUP_DEVICES_MOUNTS:
+        /* append a menu separator */
+        item = gtk_separator_menu_item_new ();
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+        gtk_widget_show (item);
+
         /* append the "Disconnect" item */
         item = gtk_image_menu_item_new_with_mnemonic (_("Disconn_ect"));
         gtk_widget_set_sensitive (item, thunar_device_can_eject (device));
         g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_shortcuts_view_eject), view);
         gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
         gtk_widget_show (item);
+
+        image = gtk_image_new_from_stock (GTK_STOCK_DISCONNECT, GTK_ICON_SIZE_MENU);
+        gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
         break;
 
       case THUNAR_SHORTCUT_GROUP_PLACES_TRASH:
@@ -1506,7 +1570,7 @@ static void
 thunar_shortcuts_view_open_clicked (ThunarShortcutsView *view)
 {
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
-  thunar_shortcuts_view_open (view, FALSE);
+  thunar_shortcuts_view_open (view, OPEN_IN_VIEW);
 }
 
 
@@ -1519,14 +1583,14 @@ thunar_shortcuts_view_poke_file_finish (ThunarBrowser *browser,
                                         gpointer       user_data)
 {
   ThunarApplication *application;
-  gboolean           new_window = GPOINTER_TO_UINT (user_data);
+  OpenTarget         open_in = GPOINTER_TO_UINT (user_data);
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (browser));
   _thunar_return_if_fail (THUNAR_IS_FILE (target_file));
 
   if (error == NULL)
     {
-      if (new_window)
+      if (open_in == OPEN_IN_WINDOW)
         {
           /* open a new window for the target folder */
           application = thunar_application_get ();
@@ -1534,6 +1598,11 @@ thunar_shortcuts_view_poke_file_finish (ThunarBrowser *browser,
                                           gtk_widget_get_screen (GTK_WIDGET (browser)), NULL);
           g_object_unref (application);
         }
+      else if (open_in == OPEN_IN_TAB)
+        {
+          /* invoke the signal to change to open folder in a new tab */
+          g_signal_emit (browser, view_signals[SHORTCUT_ACTIVATED_TAB], 0, target_file);
+        }
       else if (thunar_file_check_loaded (target_file))
         {
           /* invoke the signal to change to that folder */
@@ -1569,7 +1638,7 @@ thunar_shortcuts_view_poke_location_finish (ThunarBrowser *browser,
     }
   else
     {
-      name = thunar_g_file_get_display_name (location);
+      name = thunar_g_file_get_display_name_remote (location);
       thunar_dialogs_show_error (GTK_WIDGET (browser), error, _("Failed to open \"%s\""), name);
       g_free (name);
     }
@@ -1584,7 +1653,6 @@ thunar_shortcuts_view_poke_device_finish (ThunarBrowser *browser,
                                           GError        *error,
                                           gpointer       user_data)
 {
-  gboolean      new_window = GPOINTER_TO_UINT (user_data);
   gchar        *device_name;
   GtkTreeModel *model;
   GtkTreeModel *child_model;
@@ -1596,7 +1664,7 @@ thunar_shortcuts_view_poke_device_finish (ThunarBrowser *browser,
     {
       thunar_browser_poke_file (browser, mount_point, GTK_WIDGET (browser),
                                 thunar_shortcuts_view_poke_file_finish,
-                                GUINT_TO_POINTER (new_window));
+                                user_data);
     }
   else
     {
@@ -1616,7 +1684,7 @@ thunar_shortcuts_view_poke_device_finish (ThunarBrowser *browser,
 
 static void
 thunar_shortcuts_view_open (ThunarShortcutsView *view,
-                            gboolean             new_window)
+                            OpenTarget           open_in)
 {
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
@@ -1652,19 +1720,19 @@ thunar_shortcuts_view_open (ThunarShortcutsView *view,
 
           thunar_browser_poke_device (THUNAR_BROWSER (view), device, view,
                                       thunar_shortcuts_view_poke_device_finish,
-                                      GUINT_TO_POINTER (new_window));
+                                      GUINT_TO_POINTER (open_in));
         }
       else if (file != NULL)
         {
           thunar_browser_poke_file (THUNAR_BROWSER (view), file, view,
                                     thunar_shortcuts_view_poke_file_finish,
-                                    GUINT_TO_POINTER (new_window));
+                                    GUINT_TO_POINTER (open_in));
         }
       else if (location != NULL)
         {
           thunar_browser_poke_location (THUNAR_BROWSER (view), location, view,
                                         thunar_shortcuts_view_poke_location_finish,
-                                        GUINT_TO_POINTER (new_window));
+                                        GUINT_TO_POINTER (open_in));
         }
 
       if (file != NULL)
@@ -1684,7 +1752,16 @@ static void
 thunar_shortcuts_view_open_in_new_window_clicked (ThunarShortcutsView *view)
 {
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
-  thunar_shortcuts_view_open (view, TRUE);
+  thunar_shortcuts_view_open (view, OPEN_IN_WINDOW);
+}
+
+
+
+static void
+thunar_shortcuts_view_open_in_new_tab_clicked (ThunarShortcutsView *view)
+{
+  _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
+  thunar_shortcuts_view_open (view, OPEN_IN_TAB);
 }
 
 
@@ -1704,6 +1781,41 @@ thunar_shortcuts_view_empty_trash (ThunarShortcutsView *view)
 
 
 
+static void
+thunar_shortcuts_view_create_shortcut (ThunarShortcutsView *view)
+{
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  ThunarDevice     *device;
+  GFile            *mount_point;
+  GtkTreeModel     *shortcuts_model;
+
+  _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
+
+  /* determine the selected item */
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    {
+      /* determine the device/mount for the shortcut at the given tree iterator */
+      gtk_tree_model_get (model, &iter, THUNAR_SHORTCUTS_MODEL_COLUMN_DEVICE, &device, -1);
+      _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
+
+      /* add the mount point to the model */
+      mount_point = thunar_device_get_root (device);
+      if (mount_point != NULL)
+        {
+          shortcuts_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
+          thunar_shortcuts_model_add (THUNAR_SHORTCUTS_MODEL (shortcuts_model), NULL, mount_point);
+          g_object_unref (mount_point);
+        }
+
+      g_object_unref (G_OBJECT (device));
+    }
+}
+
+
+
 static void
 thunar_shortcuts_view_eject_finish (ThunarDevice *device,
                                     const GError *error,
