@@ -829,13 +829,14 @@ thunar_thumbnailer_queue_file (ThunarThumbnailer *thumbnailer,
   files.prev = NULL;
 
   /* queue a thumbnail request for the file */
-  return thunar_thumbnailer_queue_files (thumbnailer, &files, request);
+  return thunar_thumbnailer_queue_files (thumbnailer, FALSE, &files, request);
 }
 
 
 
 gboolean
 thunar_thumbnailer_queue_files (ThunarThumbnailer *thumbnailer,
+                                gboolean           lazy_checks,
                                 GList             *files,
                                 guint             *request)
 {
@@ -870,18 +871,21 @@ thunar_thumbnailer_queue_files (ThunarThumbnailer *thumbnailer,
   /* collect all supported files from the list that are neither in the
    * about to be queued (wait queue), nor already queued, nor already
    * processed (and awaiting to be refreshed) */
-  for (lp = g_list_last (files); lp != NULL; lp = lp->prev)
+  for (lp = files; lp != NULL; lp = lp->next)
     {
       /* get the current thumb state */
       thumb_state = thunar_file_get_thumb_state (lp->data);
 
-      /* if previously this failed, don't both to check. this saves a
-       * lot of check when resizing a window */
-      if (thumb_state == THUNAR_FILE_THUMB_STATE_NONE)
-        continue;
+      if (lazy_checks)
+        {
+          /* in lazy mode, don't both for files that have already
+           * been loaded or are not supported */
+          if (thumb_state == THUNAR_FILE_THUMB_STATE_NONE
+              || thumb_state == THUNAR_FILE_THUMB_STATE_READY)
+            continue;
+        }
 
-      /* if the state is unknown of there previously was a thumb, add
-       * to the supported list */
+      /* check if the file is supported, assume it is when the state was ready previously */
       if (thumb_state == THUNAR_FILE_THUMB_STATE_READY
           || thunar_thumbnailer_file_is_supported (thumbnailer, lp->data))
         {
@@ -907,7 +911,7 @@ thunar_thumbnailer_queue_files (ThunarThumbnailer *thumbnailer,
       mime_hints = g_new0 (const gchar *, n_items + 1);
 
       /* fill URI and MIME hint arrays with items from the wait queue */
-      for (lp = g_list_last (supported_files), n = 0; lp != NULL; lp = lp->prev, ++n)
+      for (lp = supported_files, n = 0; lp != NULL; lp = lp->next, ++n)
         {
           /* set the thumbnail state to loading */
           thunar_file_set_thumb_state (lp->data, THUNAR_FILE_THUMB_STATE_LOADING);
@@ -947,6 +951,7 @@ thunar_thumbnailer_queue_files (ThunarThumbnailer *thumbnailer,
 }
 
 
+
 void
 thunar_thumbnailer_dequeue (ThunarThumbnailer *thumbnailer,
                             guint              request)
