@@ -25,9 +25,6 @@
 
 #include <gdk/gdkkeysyms.h>
 
-#ifdef HAVE_LIBNOTIFY
-#include <thunar/thunar-notify.h>
-#endif
 #include <thunar/thunar-application.h>
 #include <thunar/thunar-clipboard-manager.h>
 #include <thunar/thunar-create-dialog.h>
@@ -39,6 +36,7 @@
 #include <thunar/thunar-marshal.h>
 #include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
+#include <thunar/thunar-stock.h>
 #include <thunar/thunar-properties-dialog.h>
 #include <thunar/thunar-shortcuts-icon-renderer.h>
 #include <thunar/thunar-simple-job.h>
@@ -149,6 +147,7 @@ static ThunarDevice            *thunar_tree_view_get_selected_device          (T
 static void                     thunar_tree_view_action_copy                  (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_create_folder         (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_cut                   (ThunarTreeView          *view);
+static void                     thunar_tree_view_action_move_to_trash         (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_delete                (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_rename                (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_eject                 (ThunarTreeView          *view);
@@ -160,11 +159,13 @@ static void                     thunar_tree_view_mount_finish                 (T
                                                                                gpointer                 user_data);
 static void                     thunar_tree_view_mount                        (ThunarTreeView          *view,
                                                                                gboolean                 open_after_mounting,
-                                                                               gboolean                 open_in_new_window);
+                                                                               guint                    open_in);
 static void                     thunar_tree_view_action_open                  (ThunarTreeView          *view);
 static void                     thunar_tree_view_open_selection               (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_open_in_new_window    (ThunarTreeView          *view);
+static void                     thunar_tree_view_action_open_in_new_tab       (ThunarTreeView          *view);
 static void                     thunar_tree_view_open_selection_in_new_window (ThunarTreeView          *view);
+static void                     thunar_tree_view_open_selection_in_new_tab    (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_paste_into_folder     (ThunarTreeView          *view);
 static void                     thunar_tree_view_action_properties            (ThunarTreeView          *view);
 static GClosure                *thunar_tree_view_new_files_closure            (ThunarTreeView          *view);
@@ -188,7 +189,7 @@ static void                     thunar_tree_view_expand_timer_destroy         (g
 static ThunarTreeViewMountData *thunar_tree_view_mount_data_new               (ThunarTreeView          *view,
                                                                                GtkTreePath             *path,
                                                                                gboolean                 open_after_mounting,
-                                                                               gboolean                 open_in_new_window);
+                                                                               guint                    open_in);
 static void                     thunar_tree_view_mount_data_free              (ThunarTreeViewMountData *data);
 static gboolean                 thunar_tree_view_get_show_hidden              (ThunarTreeView          *view);
 static void                     thunar_tree_view_set_show_hidden              (ThunarTreeView          *view,
@@ -249,12 +250,19 @@ struct _ThunarTreeView
   gint                    expand_timer_id;
 };
 
+enum
+{
+  OPEN_IN_VIEW,
+  OPEN_IN_WINDOW,
+  OPEN_IN_TAB
+};
+
 struct _ThunarTreeViewMountData
 {
   ThunarTreeView *view;
   GtkTreePath    *path;
   gboolean        open_after_mounting;
-  gboolean        open_in_new_window;
+  guint           open_in;
 };
 
 
@@ -711,8 +719,9 @@ thunar_tree_view_button_press_event (GtkWidget      *widget,
               result = TRUE;
             }
         }
-      else if ((event->button == 1 || event->button == 2) && event->type == GDK_BUTTON_PRESS
-            && (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
+      else if ((event->button == 1 || event->button == 2)
+               && event->type == GDK_BUTTON_PRESS
+               && (event->state & gtk_accelerator_get_default_mod_mask ()) == 0)
         {
           /* remember the button as pressed and handled it in the release handler */
           view->pressed_button = event->button;
@@ -732,15 +741,24 @@ thunar_tree_view_button_release_event (GtkWidget      *widget,
                                        GdkEventButton *event)
 {
   ThunarTreeView *view = THUNAR_TREE_VIEW (widget);
+  gboolean        in_tab;
 
   /* check if we have an event matching the pressed button state */
   if (G_LIKELY (view->pressed_button == (gint) event->button))
     {
       /* check if we should simply open or open in new window */
       if (G_LIKELY (event->button == 1))
-        thunar_tree_view_action_open (view);
+        {
+          thunar_tree_view_action_open (view);
+        }
       else if (G_UNLIKELY (event->button == 2))
-        thunar_tree_view_action_open_in_new_window (view);
+        {
+          g_object_get (view->preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+          if (in_tab)
+            thunar_tree_view_action_open_in_new_tab (view);
+          else
+            thunar_tree_view_action_open_in_new_window (view);
+        }
     }
 
   /* reset the pressed button state */
@@ -957,8 +975,6 @@ thunar_tree_view_row_activated (GtkTreeView       *tree_view,
                                 GtkTreePath       *path,
                                 GtkTreeViewColumn *column)
 {
-  ThunarTreeView *view = THUNAR_TREE_VIEW (tree_view);
-
   /* call the parent's "row-activated" handler */
   if (GTK_TREE_VIEW_CLASS (thunar_tree_view_parent_class)->row_activated != NULL)
     (*GTK_TREE_VIEW_CLASS (thunar_tree_view_parent_class)->row_activated) (tree_view, path, column);
@@ -968,9 +984,6 @@ thunar_tree_view_row_activated (GtkTreeView       *tree_view,
     gtk_tree_view_collapse_row (tree_view, path);
   else
     gtk_tree_view_expand_row (tree_view, path, FALSE);
-
-  /* ...open the selected folder */
-  thunar_tree_view_action_open (view);
 }
 
 
@@ -1000,7 +1013,7 @@ thunar_tree_view_test_expand_row (GtkTreeView *tree_view,
           expandable = FALSE;
 
           /* allocate a mount data struct */
-          data = thunar_tree_view_mount_data_new (view, path, FALSE, FALSE);
+          data = thunar_tree_view_mount_data_new (view, path, FALSE, OPEN_IN_VIEW);
 
           /* allocate a GTK+ mount operation */
           window = gtk_widget_get_toplevel (GTK_WIDGET (view));
@@ -1052,12 +1065,12 @@ thunar_tree_view_delete_selected_files (ThunarTreeView *view)
   /* Check if there is a user defined accelerator for the delete action,
    * if there is, skip events from the hard-coded keys which are set in
    * the class of the standard view. See bug #4173. */
-  if (gtk_accel_map_lookup_entry ("<Actions>/ThunarStandardView/delete", &key)
+  if (gtk_accel_map_lookup_entry ("<Actions>/ThunarStandardView/move-to-trash", &key)
       && (key.accel_key != 0 || key.accel_mods != 0))
     return FALSE;
 
   /* ask the user whether to delete the folder... */
-  thunar_tree_view_action_delete (view);
+  thunar_tree_view_action_move_to_trash (view);
 
   /* ...and we're done */
   return TRUE;
@@ -1104,9 +1117,16 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
   /* set the stock icon */
   image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+  
+  /* append the "Open in New Tab" menu action */
+  item = gtk_image_menu_item_new_with_mnemonic (_("Open in New _Tab"));
+  g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_open_in_new_tab), view);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+  gtk_widget_set_sensitive (item, (file != NULL || device != NULL));
+  gtk_widget_show (item);
 
   /* append the "Open in New Window" menu action */
-  item = gtk_image_menu_item_new_with_mnemonic (_("Open in New Window"));
+  item = gtk_image_menu_item_new_with_mnemonic (_("Open in New _Window"));
   g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_open_in_new_window), view);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   gtk_widget_set_sensitive (item, (file != NULL || device != NULL));
@@ -1238,6 +1258,26 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
           /* determine the parent file (required to determine "Delete" sensitivity) */
           parent_file = thunar_file_get_parent (file, NULL);
 
+          /* append a separator item */
+          item = gtk_separator_menu_item_new ();
+          gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+          gtk_widget_show (item);
+
+          if (thunar_g_vfs_is_uri_scheme_supported ("trash")
+              && thunar_file_can_be_trashed (file))
+            {
+              /* append the "Move to Tash" menu action */
+              item = gtk_image_menu_item_new_with_mnemonic (_("Mo_ve to Trash"));
+              g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_move_to_trash), view);
+              gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+              gtk_widget_set_sensitive (item, (parent_file != NULL && thunar_file_is_writable (parent_file)));
+              gtk_widget_show (item);
+
+              /* set the stock icon */
+              image = gtk_image_new_from_stock (THUNAR_STOCK_TRASH_FULL, GTK_ICON_SIZE_MENU);
+              gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+            }
+
           /* append the "Delete" menu action */
           item = gtk_image_menu_item_new_with_mnemonic (_("_Delete"));
           g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_tree_view_action_delete), view);
@@ -1635,7 +1675,7 @@ thunar_tree_view_action_cut (ThunarTreeView *view)
 
 
 static void
-thunar_tree_view_action_delete (ThunarTreeView *view)
+thunar_tree_view_action_move_to_trash (ThunarTreeView *view)
 {
   ThunarApplication *application;
   ThunarFile        *file;
@@ -1669,6 +1709,36 @@ thunar_tree_view_action_delete (ThunarTreeView *view)
 
 
 
+static void
+thunar_tree_view_action_delete (ThunarTreeView *view)
+{
+  ThunarApplication *application;
+  ThunarFile        *file;
+  GList              file_list;
+
+  _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
+
+  /* determine the selected file */
+  file = thunar_tree_view_get_selected_file (view);
+  if (G_LIKELY (file != NULL))
+    {
+      /* fake a file list */
+      file_list.data = file;
+      file_list.next = NULL;
+      file_list.prev = NULL;
+
+      /* delete the file */
+      application = thunar_application_get ();
+      thunar_application_unlink_files (application, GTK_WIDGET (view), &file_list, TRUE);
+      g_object_unref (G_OBJECT (application));
+
+      /* release the file */
+      g_object_unref (G_OBJECT (file));
+    }
+}
+
+
+
 static void
 thunar_tree_view_rename_error (ExoJob         *job,
                                GError         *error,
@@ -1869,7 +1939,7 @@ static void
 thunar_tree_view_action_mount (ThunarTreeView *view)
 {
   _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
-  thunar_tree_view_mount (view, FALSE, FALSE);
+  thunar_tree_view_mount (view, FALSE, OPEN_IN_VIEW);
 }
 
 
@@ -1895,10 +1965,20 @@ thunar_tree_view_mount_finish (ThunarDevice *device,
     {
       if (G_LIKELY (data->open_after_mounting))
         {
-          if (data->open_in_new_window)
-            thunar_tree_view_open_selection_in_new_window (data->view);
-          else
-            thunar_tree_view_open_selection (data->view);
+          switch (data->open_in)
+            {
+            case OPEN_IN_WINDOW:
+              thunar_tree_view_open_selection_in_new_window (data->view);
+              break;
+            
+            case OPEN_IN_TAB:
+              thunar_tree_view_open_selection_in_new_tab (data->view);
+              break;
+            
+            default:
+              thunar_tree_view_open_selection (data->view);
+              break;
+            }
         }
       else if (data->path != NULL)
         {
@@ -1914,7 +1994,7 @@ thunar_tree_view_mount_finish (ThunarDevice *device,
 static void
 thunar_tree_view_mount (ThunarTreeView *view,
                         gboolean        open_after_mounting,
-                        gboolean        open_in_new_window)
+                        guint           open_in)
 {
   ThunarTreeViewMountData *data;
   GMountOperation         *mount_operation;
@@ -1931,8 +2011,7 @@ thunar_tree_view_mount (ThunarTreeView *view,
       if (!thunar_device_is_mounted (device))
         {
           /* allocate mount data */
-          data = thunar_tree_view_mount_data_new (view, NULL, open_after_mounting,
-                                                  open_in_new_window);
+          data = thunar_tree_view_mount_data_new (view, NULL, open_after_mounting, open_in);
 
           /* allocate a GTK+ mount operation */
           window = gtk_widget_get_toplevel (GTK_WIDGET (view));
@@ -1974,7 +2053,7 @@ thunar_tree_view_action_open (ThunarTreeView *view)
       if (thunar_device_is_mounted (device))
         thunar_tree_view_open_selection (view);
       else
-        thunar_tree_view_mount (view, TRUE, FALSE);
+        thunar_tree_view_mount (view, TRUE, OPEN_IN_TAB);
 
       g_object_unref (device);
     }
@@ -2023,7 +2102,7 @@ thunar_tree_view_action_open_in_new_window (ThunarTreeView *view)
       if (thunar_device_is_mounted (device))
         thunar_tree_view_open_selection_in_new_window (view);
       else
-        thunar_tree_view_mount (view, TRUE, FALSE);
+        thunar_tree_view_mount (view, TRUE, OPEN_IN_WINDOW);
 
       g_object_unref (device);
     }
@@ -2036,6 +2115,36 @@ thunar_tree_view_action_open_in_new_window (ThunarTreeView *view)
 
 
 
+static void
+thunar_tree_view_action_open_in_new_tab (ThunarTreeView *view)
+{
+  ThunarFile   *file;
+  ThunarDevice *device;
+
+  _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
+
+  /* determine the selected device and file */
+  device = thunar_tree_view_get_selected_device (view);
+  file = thunar_tree_view_get_selected_file (view);
+
+  if (device != NULL)
+    {
+      if (thunar_device_is_mounted (device))
+        thunar_tree_view_open_selection_in_new_tab (view);
+      else
+        thunar_tree_view_mount (view, TRUE, OPEN_IN_TAB);
+
+      g_object_unref (device);
+    }
+  else if (file != NULL)
+    {
+      thunar_tree_view_open_selection_in_new_tab (view);
+      g_object_unref (file);
+    }
+}
+
+
+
 static void
 thunar_tree_view_open_selection_in_new_window (ThunarTreeView *view)
 {
@@ -2059,6 +2168,25 @@ thunar_tree_view_open_selection_in_new_window (ThunarTreeView *view)
 
 
 
+static void
+thunar_tree_view_open_selection_in_new_tab (ThunarTreeView *view)
+{
+  ThunarFile *file;
+
+  _thunar_return_if_fail (THUNAR_IS_TREE_VIEW (view));
+
+  /* determine the selected file */
+  file = thunar_tree_view_get_selected_file (view);
+  if (G_LIKELY (file != NULL))
+    {
+      /* open a new tab for the selected folder */
+      thunar_navigator_open_new_tab (THUNAR_NAVIGATOR (view), file);
+      g_object_unref (file);
+    }
+}
+
+
+
 static void
 thunar_tree_view_action_paste_into_folder (ThunarTreeView *view)
 {
@@ -2162,13 +2290,15 @@ thunar_tree_view_visible_func (ThunarTreeModel *model,
                                ThunarFile      *file,
                                gpointer         user_data)
 {
-  ThunarTreeView *view = THUNAR_TREE_VIEW (user_data);
+  ThunarTreeView *view;
   gboolean        visible = TRUE;
 
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_TREE_MODEL (model), FALSE);
+  _thunar_return_val_if_fail (THUNAR_IS_TREE_VIEW (user_data), FALSE);
 
   /* if show_hidden is TRUE, nothing is filtered */
+  view = THUNAR_TREE_VIEW (user_data);
   if (G_LIKELY (!view->show_hidden))
     {
       /* we display all non-hidden file and hidden files that are ancestors of the current directory */
@@ -2438,7 +2568,7 @@ static ThunarTreeViewMountData *
 thunar_tree_view_mount_data_new (ThunarTreeView *view,
                                  GtkTreePath    *path,
                                  gboolean        open_after_mounting,
-                                 gboolean        open_in_new_window)
+                                 guint           open_in)
 {
   ThunarTreeViewMountData *data;
 
@@ -2446,7 +2576,7 @@ thunar_tree_view_mount_data_new (ThunarTreeView *view,
   data->path = path == NULL ? NULL : gtk_tree_path_copy (path);
   data->view = g_object_ref (view);
   data->open_after_mounting = open_after_mounting;
-  data->open_in_new_window = open_in_new_window;
+  data->open_in = open_in;
 
   return data;
 }
